@@ -276,7 +276,8 @@ UNSTABLE_STATES = (DEPLOYING, DEPLOYWAIT, CLEANING, CLEANWAIT, VERIFYING,
 """States that can be changed without external request."""
 
 STUCK_STATES_TREATED_AS_FAIL = (DEPLOYING, CLEANING, VERIFYING, INSPECTING,
-                                ADOPTING, RESCUING, UNRESCUING, DELETING)
+                                ADOPTING, RESCUING, UNRESCUING, DELETING,
+                                SERVICING)
 """States that cannot be resumed once a conductor dies.
 
 If a node gets stuck with one of these states for some reason
@@ -665,3 +666,6 @@ machine.add_transition(SERVICEFAIL, SERVICING, 'service')
 
 # A node in service fail can be rescued
 machine.add_transition(SERVICEFAIL, RESCUING, 'rescue')
+
+# A no in service fail may be deleted.
+machine.add_transition(SERVICEFAIL, DELETING, 'delete')
