@@ -338,6 +338,10 @@ class TaskManager(object):
         else:
             stop_after = tenacity.stop_after_attempt(1)
 
+        max_lock_time = \
+            CONF.conductor.node_locked_retry_interval * \
+            CONF.conductor.node_locked_retry_attempts
+
         # NodeLocked exceptions can be annoying. Let's try to alleviate
         # some of that pain by retrying our lock attempts.
         @tenacity.retry(
@@ -347,6 +351,13 @@ class TaskManager(object):
                 CONF.conductor.node_locked_retry_interval),
             reraise=True)
         def reserve_node():
+            if self._debug_timer.elapsed() > max_lock_time:
+                LOG.warning('We have exceeded the normal maximum time window '
+                            'to complete a node lock attempting to reserve '
+                            'node %(node)s for purpose %(purpose)s. At '
+                            '%(time).2f seconds.',
+                            {'node': self.node_id, 'purpose': self._purpose,
+                             'time': self._debug_timer.elapsed()})
             self.node = objects.Node.reserve(self.context, CONF.host,
                                              self.node_id)
             LOG.debug("Node %(node)s successfully reserved for %(purpose)s "
@@ -527,7 +538,8 @@ class TaskManager(object):
             self.release_resources()
 
     def process_event(self, event, callback=None, call_args=None,
-                      call_kwargs=None, err_handler=None, target_state=None):
+                      call_kwargs=None, err_handler=None, target_state=None,
+                      last_error=None):
         """Process the given event for the task's current state.
 
         :param event: the name of the event to process
@@ -540,6 +552,8 @@ class TaskManager(object):
                 prev_target_state)
         :param target_state: if specified, the target provision state for the
                node. Otherwise, use the target state from the fsm
+        :param last_error: last error to set on the node together with
+               the state transition.
         :raises: InvalidState if the event is not allowed by the associated
                  state machine
         """
@@ -572,13 +586,15 @@ class TaskManager(object):
 
         # set up the async worker
         if callback:
-            # clear the error if we're going to start work in a callback
-            self.node.last_error = None
+            # update the error if we're going to start work in a callback
+            self.node.last_error = last_error
             if call_args is None:
                 call_args = ()
             if call_kwargs is None:
                 call_kwargs = {}
             self.spawn_after(callback, *call_args, **call_kwargs)
+        elif last_error is not None:
+            self.node.last_error = last_error
 
         # publish the state transition by saving the Node
         self.node.save()
