@@ -170,6 +170,13 @@ opts = [
                       'ramdisk doing the cleaning. If the timeout is reached '
                       'the node will be put in the "clean failed" provision '
                       'state. Set to 0 to disable timeout.')),
+    cfg.IntOpt('service_callback_timeout',
+               default=1800,
+               min=0,
+               help=_('Timeout (seconds) to wait for a callback from the '
+                      'ramdisk doing the servicing. If the timeout is reached '
+                      'the node will be put in the "service failed" provision '
+                      'state. Set to 0 to disable timeout.')),
     cfg.IntOpt('rescue_callback_timeout',
                default=1800,
                min=0,
@@ -487,7 +494,7 @@ opts = [
                        '[conductor]disable_deep_image_inspection to be set '
                        'to False.')),
     cfg.ListOpt('permitted_image_formats',
-                default=['raw', 'qcow2', 'iso'],
+                default=['raw', 'gpt', 'qcow2', 'iso'],
                 mutable=True,
                 help=_('The supported list of image formats which are '
                        'permitted for deployment with Ironic. If an image '
@@ -515,6 +522,15 @@ opts = [
                        'functionality by setting this option to True will '
                        'create a more secure environment, however it may '
                        'break users in an unexpected fashion.')),
+    cfg.BoolOpt('disable_zstandard_decompression',
+                default=False,
+                mutable=False,
+                help=_('Option to enable disabling transparent decompression '
+                       'of files which are compressed with Zstandard '
+                       'compression. This option is provided should operators '
+                       'wish to disable this functionality, otherwise it is '
+                       'automaticly applied by the conductor should a '
+                       'compressed artifact be detected.')),
 ]
 
 
