@@ -63,11 +63,18 @@ api_opts = [
                  ('http_basic', _('HTTP basic authentication'))],
         help=_('Authentication strategy used by ironic-api. "noauth" should '
                'not be used in a production environment because all '
-               'authentication will be disabled.')),
+               'authentication will be disabled creating insecure '
+               'operating conditions.')),
     cfg.StrOpt('http_basic_auth_user_file',
                default='/etc/ironic/htpasswd',
                help=_('Path to Apache format user authentication file used '
                       'when auth_strategy=http_basic')),
+    cfg.BoolOpt(
+        'ignore_project_check_for_admin_tasks',
+        default=True,
+        help=_('If True, allows admin tasks to access image without'
+               'matching project_id')
+    ),
     cfg.BoolOpt('debug_tracebacks_in_api',
                 default=False,
                 help=_('Return server tracebacks in the API response for any '
@@ -81,6 +88,11 @@ api_opts = [
                mutable=True,
                help=_('Resource class to use for new nodes when no resource '
                       'class is provided in the creation request.')),
+    cfg.StrOpt('default_conductor_group',
+               mutable=True,
+               default="",
+               help=_('The conductor_group to use for new nodes when no '
+                      'conductor_group was defined in the creation request.')),
 ]
 
 driver_opts = [
@@ -115,6 +127,11 @@ driver_opts = [
                 help=_ENABLED_IFACE_HELP.format('deploy')),
     cfg.StrOpt('default_deploy_interface',
                help=_DEFAULT_IFACE_HELP.format('deploy')),
+    cfg.ListOpt('enabled_firmware_interfaces',
+                default=['no-firmware'],
+                help=_ENABLED_IFACE_HELP.format('firmware')),
+    cfg.StrOpt('default_firmware_interface',
+               help=_DEFAULT_IFACE_HELP.format('firmware')),
     cfg.ListOpt('enabled_inspect_interfaces',
                 default=['no-inspect', 'redfish'],
                 help=_ENABLED_IFACE_HELP.format('inspect')),
@@ -216,7 +233,7 @@ image_opts = [
                                     'common/isolinux_config.template'),
                help=_('Template file for isolinux configuration file.')),
     cfg.StrOpt('grub_config_path',
-               default='/boot/grub/grub.cfg',
+               default='EFI/BOOT/grub.cfg',
                help=_('GRUB2 configuration file location on the UEFI ISO '
                       'images produced by ironic. The default value is '
                       'usually incorrect and should not be relied on. '
@@ -345,9 +362,7 @@ service_opts = [
                help=_('Name of this node. This can be an opaque identifier. '
                       'It is not necessarily a hostname, FQDN, or IP address. '
                       'However, the node name must be valid within '
-                      'an AMQP key, and if using ZeroMQ (will be removed in '
-                      'the Stein release), a valid hostname, FQDN, '
-                      'or IP address.')),
+                      'an AMQP key.')),
     cfg.StrOpt('pin_release_version',
                choices=versions.RELEASE_VERSIONS_DESCS,
                mutable=True,
@@ -392,13 +407,17 @@ service_opts = [
                help=_('Number of retries to hold onto the worker before '
                       'failing or returning the thread to the pool if '
                       'the conductor can automatically retry.')),
+    cfg.IntOpt('drain_shutdown_timeout',
+               mutable=True,
+               default=1800,
+               help=_('Timeout (seconds) after which a server will exit '
+                      'from a drain shutdown. Drain shutdowns are '
+                      'triggered by sending the signal SIGUSR2. '
+                      'Zero value means shutdown will never be triggered by '
+                      'a timeout.')),
 ]
 
 utils_opts = [
-    cfg.StrOpt('rootwrap_config',
-               default="/etc/ironic/rootwrap.conf",
-               help=_('Path to the rootwrap configuration file to use for '
-                      'running commands as root.')),
     cfg.StrOpt('tempdir',
                default=tempfile.gettempdir(),
                sample_default=tempfile.gettempdir(),
@@ -426,8 +445,47 @@ webserver_opts = [
                       'Defaults to True.')),
     cfg.IntOpt('webserver_connection_timeout',
                default=60,
-               help=_('Connection timeout when accessing remote web servers '
-                      'with images.')),
+               help=_('Connection timeout when accessing/interacting with '
+                      'remote web servers with images or other artifacts '
+                      'being accessed. An excessive value here is not '
+                      'advisable as excessive requests to an unreachable '
+                      'endpoint can result in Ironic service resources '
+                      'being consumed waiting for the connection to '
+                      'timeout.')),
+]
+
+rbac_opts = [
+    cfg.BoolOpt('rbac_service_role_elevated_access',
+                default=False,
+                help=_('Enable elevated access for users with service role '
+                       'belonging to the \'rbac_service_project_name\' '
+                       'project when using default policy. The default '
+                       'setting of disabled causes all service role '
+                       'requests to be scoped to the project the service '
+                       'account belongs to.')),
+    cfg.StrOpt('rbac_service_project_name',
+               default='service',
+               help=_('The project name utilized for Role Based Access '
+                      'Control checks for the reserved `service` project. '
+                      'This project is utilized for services to have '
+                      'accounts for cross-service communication. Often '
+                      'these accounts require higher levels of access, and '
+                      'effectively this permits accounts from the service '
+                      'to not be restricted to project scoping '
+                      'of responses. i.e. The service project user with a '
+                      '`service` role will be able to see nodes across all '
+                      'projects, similar to System scoped access. If not '
+                      'set to a value, and all service role access will '
+                      'be filtered matching an `owner` or `lessee`, if '
+                      'applicable. If an operator wishes to make behavior '
+                      'visible for all service role users across '
+                      'all projects, then a custom policy must be used '
+                      'to override the default "service_role" rule. '
+                      'It should be noted that the value of "service" '
+                      'is a default convention for OpenStack deployments, '
+                      'but the requisite access and details around '
+                      'end configuration are largely up to an operator '
+                      'if they are doing an OpenStack deployment manually.')),
 ]
 
 
@@ -446,6 +504,7 @@ def list_opts():
         service_opts,
         utils_opts,
         webserver_opts,
+        rbac_opts
     ]
     full_opt_list = []
     for options in _default_opt_lists:
@@ -467,3 +526,4 @@ def register_opts(conf):
     conf.register_opts(service_opts)
     conf.register_opts(utils_opts)
     conf.register_opts(webserver_opts)
+    conf.register_opts(rbac_opts)
