@@ -19,16 +19,18 @@ SQLAlchemy models for baremetal data.
 """
 
 from os import path
+from typing import List
 from urllib import parse as urlparse
 
 from oslo_db import options as db_options
 from oslo_db.sqlalchemy import models
 from oslo_db.sqlalchemy import types as db_types
+from sqlalchemy.ext.associationproxy import association_proxy
 from sqlalchemy import Boolean, Column, DateTime, false, Index
 from sqlalchemy import ForeignKey, Integer
 from sqlalchemy import schema, String, Text
-from sqlalchemy.ext.declarative import declarative_base
 from sqlalchemy import orm
+from sqlalchemy.orm import declarative_base
 
 from ironic.common import exception
 from ironic.common.i18n import _
@@ -116,8 +118,8 @@ class ConductorHardwareInterfaces(Base):
     default = Column(Boolean, default=False, nullable=False)
 
 
-class Node(Base):
-    """Represents a bare metal node."""
+class NodeBase(Base):
+    """Represents a base bare metal node."""
 
     __tablename__ = 'nodes'
     __table_args__ = (
@@ -125,6 +127,15 @@ class Node(Base):
         schema.UniqueConstraint('instance_uuid',
                                 name='uniq_nodes0instance_uuid'),
         schema.UniqueConstraint('name', name='uniq_nodes0name'),
+        Index('owner_idx', 'owner'),
+        Index('lessee_idx', 'lessee'),
+        Index('driver_idx', 'driver'),
+        Index('provision_state_idx', 'provision_state'),
+        Index('reservation_idx', 'reservation'),
+        Index('conductor_group_idx', 'conductor_group'),
+        Index('resource_class_idx', 'resource_class'),
+        Index('shard_idx', 'shard'),
+        Index('parent_node_idx', 'parent_node'),
         table_args())
     id = Column(Integer, primary_key=True)
     uuid = Column(String(36))
@@ -189,6 +200,7 @@ class Node(Base):
     boot_interface = Column(String(255), nullable=True)
     console_interface = Column(String(255), nullable=True)
     deploy_interface = Column(String(255), nullable=True)
+    firmware_interface = Column(String(255), nullable=True)
     inspect_interface = Column(String(255), nullable=True)
     management_interface = Column(String(255), nullable=True)
     network_interface = Column(String(255), nullable=True)
@@ -201,6 +213,39 @@ class Node(Base):
     storage_interface = Column(String(255), nullable=True)
     power_interface = Column(String(255), nullable=True)
     vendor_interface = Column(String(255), nullable=True)
+    boot_mode = Column(String(16), nullable=True)
+    secure_boot = Column(Boolean, nullable=True)
+    shard = Column(String(255), nullable=True)
+    parent_node = Column(String(36), nullable=True)
+    service_step = Column(db_types.JsonEncodedDict)
+    disable_power_off = Column(Boolean, nullable=True, default=False,
+                               server_default=false())
+
+
+class Node(NodeBase):
+    """Represents a bare metal node."""
+
+    # NOTE(TheJulia): The purpose of the delineation between NodeBase and Node
+    # is to facilitate a hard delineation for queries where we do not need to
+    # populate additional information needlessly which would normally populate
+    # from the access of the property. In this case, Traits and Tags.
+    # The other reason we do this, is because these are generally "joined"
+    # data structures, we cannot de-duplicate node objects with unhashable dict
+    # data structures.
+
+    # NOTE(TheJulia): The choice of selectin lazy population is intentional
+    # as it causes a subselect to occur, skipping the need for deduplication
+    # in general. This puts a slightly higher query load on the DB server, but
+    # means *far* less gets shipped over the wire in the end.
+    traits: orm.Mapped[List['NodeTrait']] = orm.relationship(  # noqa
+        "NodeTrait",
+        back_populates="node",
+        lazy="selectin")
+
+    tags: orm.Mapped[List['NodeTag']] = orm.relationship(  # noqa
+        "NodeTag",
+        back_populates="node",
+        lazy="selectin")
 
 
 class Port(Base):
@@ -210,6 +255,7 @@ class Port(Base):
     __table_args__ = (
         schema.UniqueConstraint('address', name='uniq_ports0address'),
         schema.UniqueConstraint('uuid', name='uniq_ports0uuid'),
+        schema.UniqueConstraint('name', name='uniq_ports0name'),
         table_args())
     id = Column(Integer, primary_key=True)
     uuid = Column(String(36))
@@ -222,6 +268,16 @@ class Port(Base):
     internal_info = Column(db_types.JsonEncodedDict)
     physical_network = Column(String(64), nullable=True)
     is_smartnic = Column(Boolean, nullable=True, default=False)
+    name = Column(String(255), nullable=True)
+
+    _node_uuid = orm.relationship(
+        "Node",
+        viewonly=True,
+        primaryjoin="(Node.id == Port.node_id)",
+        lazy="selectin",
+    )
+    node_uuid = association_proxy(
+        "_node_uuid", "uuid", creator=lambda _i: Node(uuid=_i))
 
 
 class Portgroup(Base):
@@ -244,6 +300,15 @@ class Portgroup(Base):
     mode = Column(String(255))
     properties = Column(db_types.JsonEncodedDict)
 
+    _node_uuid = orm.relationship(
+        "Node",
+        viewonly=True,
+        primaryjoin="(Node.id == Portgroup.node_id)",
+        lazy="selectin",
+    )
+    node_uuid = association_proxy(
+        "_node_uuid", "uuid", creator=lambda _i: Node(uuid=_i))
+
 
 class NodeTag(Base):
     """Represents a tag of a bare metal node."""
@@ -258,7 +323,6 @@ class NodeTag(Base):
 
     node = orm.relationship(
         "Node",
-        backref='tags',
         primaryjoin='and_(NodeTag.node_id == Node.id)',
         foreign_keys=node_id
     )
@@ -315,7 +379,6 @@ class NodeTrait(Base):
     trait = Column(String(255), primary_key=True, nullable=False)
     node = orm.relationship(
         "Node",
-        backref='traits',
         primaryjoin='and_(NodeTrait.node_id == Node.id)',
         foreign_keys=node_id
     )
@@ -330,6 +393,15 @@ class BIOSSetting(Base):
                      primary_key=True, nullable=False)
     name = Column(String(255), primary_key=True, nullable=False)
     value = Column(Text, nullable=True)
+    attribute_type = Column(String(255), nullable=True)
+    allowable_values = Column(db_types.JsonEncodedList, nullable=True)
+    lower_bound = Column(Integer, nullable=True)
+    max_length = Column(Integer, nullable=True)
+    min_length = Column(Integer, nullable=True)
+    read_only = Column(Boolean, nullable=True)
+    reset_required = Column(Boolean, nullable=True)
+    unique = Column(Boolean, nullable=True)
+    upper_bound = Column(Integer, nullable=True)
 
 
 class Allocation(Base):
@@ -343,7 +415,7 @@ class Allocation(Base):
     id = Column(Integer, primary_key=True)
     uuid = Column(String(36), nullable=False)
     name = Column(String(255), nullable=True)
-    node_id = Column(Integer, ForeignKey('nodes.id'), nullable=True)
+    node_id = Column(Integer, nullable=True)
     state = Column(String(15), nullable=False)
     owner = Column(String(255), nullable=True)
     last_error = Column(Text, nullable=True)
@@ -352,8 +424,7 @@ class Allocation(Base):
     candidate_nodes = Column(db_types.JsonEncodedList)
     extra = Column(db_types.JsonEncodedDict)
     # The last conductor to handle this allocation (internal field).
-    conductor_affinity = Column(Integer, ForeignKey('conductors.id'),
-                                nullable=True)
+    conductor_affinity = Column(Integer, nullable=True)
 
 
 class DeployTemplate(Base):
@@ -368,6 +439,10 @@ class DeployTemplate(Base):
     uuid = Column(String(36))
     name = Column(String(255), nullable=False)
     extra = Column(db_types.JsonEncodedDict)
+    steps: orm.Mapped[List['DeployTemplateStep']] = orm.relationship(  # noqa
+        "DeployTemplateStep",
+        back_populates="deploy_template",
+        lazy="selectin")
 
 
 class DeployTemplateStep(Base):
@@ -388,7 +463,6 @@ class DeployTemplateStep(Base):
     priority = Column(Integer, nullable=False)
     deploy_template = orm.relationship(
         "DeployTemplate",
-        backref='steps',
         primaryjoin=(
             'and_(DeployTemplateStep.deploy_template_id == '
             'DeployTemplate.id)'),
@@ -396,6 +470,119 @@ class DeployTemplateStep(Base):
     )
 
 
+class NodeHistory(Base):
+    """Represents a history event of a bare metal node."""
+
+    __tablename__ = 'node_history'
+    __table_args__ = (
+        schema.UniqueConstraint('uuid', name='uniq_history0uuid'),
+        Index('history_node_id_idx', 'node_id'),
+        Index('history_uuid_idx', 'uuid'),
+        Index('history_conductor_idx', 'conductor'),
+        table_args())
+    id = Column(Integer, primary_key=True)
+    uuid = Column(String(36), nullable=False)
+    conductor = Column(String(255), nullable=True)
+    event_type = Column(String(255), nullable=True)
+    severity = Column(String(255), nullable=True)
+    event = Column(Text, nullable=True)
+    user = Column(String(64), nullable=True)
+    node_id = Column(Integer, ForeignKey('nodes.id'), nullable=True)
+
+
+class NodeInventory(Base):
+    """Represents an inventory of a baremetal node."""
+    __tablename__ = 'node_inventory'
+    __table_args__ = (
+        Index('inventory_node_id_idx', 'node_id'),
+        table_args())
+    id = Column(Integer, primary_key=True)
+    inventory_data = Column(db_types.JsonEncodedDict(mysql_as_long=True))
+    plugin_data = Column(db_types.JsonEncodedDict(mysql_as_long=True))
+    node_id = Column(Integer, ForeignKey('nodes.id'), nullable=True)
+
+
+class FirmwareComponent(Base):
+    """Represents the firmware information of a bare metal node."""
+    __tablename__ = "firmware_information"
+    __table_args__ = (
+        schema.UniqueConstraint(
+            'node_id', 'component',
+            name='uniq_nodecomponent0node_id0component'),
+        table_args())
+    id = Column(Integer, primary_key=True)
+    node_id = Column(Integer, ForeignKey('nodes.id'), nullable=False)
+    component = Column(String(255), nullable=False)
+    initial_version = Column(String(255), nullable=False)
+    current_version = Column(String(255), nullable=True)
+    last_version_flashed = Column(String(255), nullable=True)
+
+
+class Runbook(Base):
+    """Represents a runbook."""
+
+    __tablename__ = 'runbooks'
+    __table_args__ = (
+        schema.UniqueConstraint('uuid', name='uniq_runbooks0uuid'),
+        schema.UniqueConstraint('name', name='uniq_runbooks0name'),
+        table_args())
+    id = Column(Integer, primary_key=True)
+    uuid = Column(String(36))
+    name = Column(String(255), nullable=False)
+    public = Column(Boolean, default=False)
+    owner = Column(String(255), nullable=True)
+    disable_ramdisk = Column(Boolean, default=False)
+    extra = Column(db_types.JsonEncodedDict)
+    steps: orm.Mapped[List['RunbookStep']] = orm.relationship(  # noqa
+        "RunbookStep",
+        back_populates="runbook",
+        lazy="selectin")
+
+
+class RunbookStep(Base):
+    """Represents a deployment step in a runbook."""
+
+    __tablename__ = 'runbook_steps'
+    __table_args__ = (
+        Index('runbook_id', 'runbook_id'),
+        Index('runbook_steps_interface_idx', 'interface'),
+        Index('runbook_steps_step_idx', 'step'),
+        table_args())
+    id = Column(Integer, primary_key=True)
+    runbook_id = Column(Integer, ForeignKey('runbooks.id'), nullable=False)
+    interface = Column(String(255), nullable=False)
+    step = Column(String(255), nullable=False)
+    args = Column(db_types.JsonEncodedDict, nullable=False)
+    order = Column(Integer, nullable=False)
+    runbook = orm.relationship(
+        "Runbook",
+        primaryjoin=(
+            'and_(RunbookStep.runbook_id == '
+            'Runbook.id)'),
+        foreign_keys=runbook_id
+    )
+
+
+class InspectionRule(Base):
+    __tablename__ = 'inspection_rules'
+    __table_args__ = (
+        schema.UniqueConstraint('uuid', name='uniq_inspection_rules0uuid'),
+        Index('inspection_rule_scope_idx', 'scope'),
+        Index('inspection_rule_phase_idx', 'phase'),
+        table_args())
+    id = Column(Integer, primary_key=True)
+    uuid = Column(String(36), nullable=False)
+    priority = Column(Integer, nullable=False, default=0)
+    description = Column(String(255), nullable=True)
+    scope = Column(String(255), nullable=True)
+    sensitive = Column(Boolean, default=False)
+    phase = Column(String(16), nullable=True, default='main')
+    conditions = Column(db_types.JsonEncodedList(mysql_as_long=True),
+                        nullable=True)
+    actions = Column(db_types.JsonEncodedList(mysql_as_long=True),
+                     nullable=False)
+
+
 def get_class(model_name):
     """Returns the model class with the specified name.
 
