@@ -37,8 +37,6 @@ import tempfile
 import time
 
 from eventlet.green import subprocess as green_subprocess
-from ironic_lib import metrics_utils
-from ironic_lib import utils as ironic_utils
 from oslo_concurrency import processutils
 from oslo_log import log as logging
 from oslo_utils import excutils
@@ -47,6 +45,7 @@ from oslo_utils import strutils
 from ironic.common import boot_devices
 from ironic.common import exception
 from ironic.common.i18n import _
+from ironic.common import metrics_utils
 from ironic.common import states
 from ironic.common import utils
 from ironic.conductor import task_manager
@@ -741,6 +740,9 @@ def _set_and_wait(task, power_action, driver_info, timeout=None):
         cmd_name = "soft"
         target_state = states.POWER_OFF
         timeout = timeout or CONF.conductor.soft_power_off_timeout
+    elif power_action == states.REBOOT:
+        cmd_name = "reset"
+        target_state = states.POWER_ON
 
     # NOTE(sambetts): Retries for ipmi power action failure will be handled by
     # the _exec_ipmitool function, so no need to wrap this call in its own
@@ -757,6 +759,10 @@ def _set_and_wait(task, power_action, driver_info, timeout=None):
                     "with error: %(error)s.",
                     {'node_id': driver_info['uuid'], 'cmd': cmd, 'error': e})
         raise exception.IPMIFailure(cmd=cmd)
+    if power_action == states.REBOOT:
+        LOG.debug('Sleeping 15 seconds to give the node %s a chance to power '
+                  'off before checking its state', task.node.uuid)
+        time.sleep(15)
     return cond_utils.node_wait_for_power_state(task, target_state,
                                                 timeout=timeout)
 
@@ -1086,6 +1092,11 @@ class IPMIPower(base.PowerInterface):
         elif power_state == states.SOFT_POWER_OFF:
             _soft_power_off(task, driver_info, timeout=timeout)
         elif power_state == states.SOFT_REBOOT:
+            if task.node.disable_power_off:
+                # There is no way to implement this in ipmitool, apparently
+                raise exception.UnsupportedHardwareFeature(
+                    node=task.node.uuid,
+                    feature=states.SOFT_POWER_OFF)
             _soft_power_off(task, driver_info, timeout=timeout)
             driver_utils.ensure_next_boot_device(task, driver_info)
             _power_on(task, driver_info, timeout=timeout)
@@ -1111,6 +1122,10 @@ class IPMIPower(base.PowerInterface):
 
         """
         driver_info = _parse_driver_info(task.node)
+        if task.node.disable_power_off:
+            driver_utils.ensure_next_boot_device(task, driver_info)
+            _set_and_wait(task, states.REBOOT, driver_info, timeout=timeout)
+            return
         # NOTE(jlvillal): Some BMCs will error if setting power state to off if
         # the node is already turned off.
         current_status = _power_status(driver_info)
@@ -1591,6 +1606,14 @@ class IPMIConsole(base.ConsoleInterface):
 class IPMIShellinaboxConsole(IPMIConsole):
     """A ConsoleInterface that uses ipmitool and shellinabox."""
 
+    # TODO(TheJulia): This interface is deprecated due to the shellinabox
+    # project being abandoned. It should be removed after the release of
+    # 2025.2 in advance of 2026.1's release.
+    # https://github.com/shellinabox/shellinabox
+    # https://github.com/shellinabox/shellinabox/issues/531
+
+    supported = False
+
     def _get_ipmi_cmd(self, driver_info, pw_file):
         """Get ipmi command for ipmitool usage.
 
@@ -1644,8 +1667,7 @@ class IPMIShellinaboxConsole(IPMIConsole):
         try:
             console_utils.stop_shellinabox_console(task.node.uuid)
         finally:
-            ironic_utils.unlink_without_raise(
-                _console_pwfile_path(task.node.uuid))
+            utils.unlink_without_raise(_console_pwfile_path(task.node.uuid))
         _release_allocated_port(task)
 
     def _exec_stop_console(self, driver_info):
@@ -1711,8 +1733,7 @@ class IPMISocatConsole(IPMIConsole):
         try:
             console_utils.stop_socat_console(task.node.uuid)
         finally:
-            ironic_utils.unlink_without_raise(
-                _console_pwfile_path(task.node.uuid))
+            utils.unlink_without_raise(_console_pwfile_path(task.node.uuid))
         self._exec_stop_console(driver_info)
         _release_allocated_port(task)
 
