@@ -13,13 +13,13 @@
 
 from urllib.parse import urlparse
 
-from ironic_lib import metrics_utils
 from oslo_log import log
-from oslo_utils import importutils
 from oslo_utils import timeutils
+import sushy
 
 from ironic.common import exception
 from ironic.common.i18n import _
+from ironic.common import metrics_utils
 from ironic.common import states
 from ironic.conductor import periodics
 from ironic.conductor import utils as manager_utils
@@ -34,8 +34,6 @@ LOG = log.getLogger(__name__)
 
 METRICS = metrics_utils.get_metrics_logger(__name__)
 
-sushy = importutils.try_import('sushy')
-
 
 class RedfishFirmware(base.FirmwareInterface):
 
@@ -48,13 +46,6 @@ class RedfishFirmware(base.FirmwareInterface):
         }
     }
 
-    def __init__(self):
-        super(RedfishFirmware, self).__init__()
-        if sushy is None:
-            raise exception.DriverLoadError(
-                driver='redfish',
-                reason=_("Unable to import the sushy library"))
-
     def get_properties(self):
         """Return the properties of the interface.
 
@@ -87,7 +78,7 @@ class RedfishFirmware(base.FirmwareInterface):
         node_id = task.node.id
         settings = []
         # NOTE(iurygregory): currently we will only retrieve BIOS and BMC
-        # firmware information trough the redfish system and manager.
+        # firmware information through the redfish system and manager.
 
         system = redfish_utils.get_system(task.node)
 
@@ -153,6 +144,9 @@ class RedfishFirmware(base.FirmwareInterface):
     @base.clean_step(priority=0, abortable=False,
                      argsinfo=_FW_SETTINGS_ARGSINFO,
                      requires_ramdisk=True)
+    @base.service_step(priority=0, abortable=False,
+                       argsinfo=_FW_SETTINGS_ARGSINFO,
+                       requires_ramdisk=True)
     @base.cache_firmware_components
     def update(self, task, settings):
         """Update the Firmware on the node using the settings for components.
@@ -200,6 +194,26 @@ class RedfishFirmware(base.FirmwareInterface):
             to be executed.
         """
         fw_upd = settings[0]
+        # NOTE(janders) try to get the collection of Systems on the BMC
+        # to determine if there may be more than one System
+        try:
+            systems_collection = redfish_utils.get_system_collection(node)
+        except exception.RedfishError as e:
+            LOG.error('Failed getting Redfish Systems Collection'
+                      ' for node %(node)s. Error %(error)s',
+                      {'node': node.uuid, 'error': e})
+            raise exception.RedfishError(error=e)
+        count = len(systems_collection.members_identities)
+        # NOTE(janders) if we see more than one System on the BMC, assume that
+        # we need to explicitly specify Target parameter when calling
+        # SimpleUpdate. This is needed for compatibility with sushy-tools
+        # in automated testing using VMs.
+        if count > 1:
+            target = node.driver_info.get('redfish_system_id')
+            targets = [target]
+        else:
+            targets = None
+
         component_url, cleanup = self._stage_firmware_file(node, fw_upd)
 
         LOG.debug('Applying new firmware %(url)s for %(component)s on node '
@@ -207,7 +221,11 @@ class RedfishFirmware(base.FirmwareInterface):
                   {'url': fw_upd['url'], 'component': fw_upd['component'],
                    'node_uuid': node.uuid})
         try:
-            task_monitor = update_service.simple_update(component_url)
+            if targets is not None:
+                task_monitor = update_service.simple_update(component_url,
+                                                            targets=targets)
+            else:
+                task_monitor = update_service.simple_update(component_url)
         except sushy.exceptions.MissingAttributeError as e:
             LOG.error('The attribute #UpdateService.SimpleUpdate is missing '
                       'on node %(node)s. Error: %(error)s',
@@ -261,8 +279,13 @@ class RedfishFirmware(base.FirmwareInterface):
 
             LOG.info('Firmware updates completed for node %(node)s',
                      {'node': node.uuid})
+            if task.node.clean_step:
+                manager_utils.notify_conductor_resume_clean(task)
+            elif task.node.service_step:
+                manager_utils.notify_conductor_resume_service(task)
+            elif task.node.deploy_step:
+                manager_utils.notify_conductor_resume_deploy(task)
 
-            manager_utils.notify_conductor_resume_clean(task)
         else:
             settings.pop(0)
             self._execute_firmware_update(node,
@@ -290,8 +313,8 @@ class RedfishFirmware(base.FirmwareInterface):
     @periodics.node_periodic(
         purpose='checking if async update of firmware component failed',
         spacing=CONF.redfish.firmware_update_fail_interval,
-        filters={'reserved': False, 'provision_state': states.CLEANFAIL,
-                 'maintenance': True},
+        filters={'reserved': False, 'provision_state_in': [states.CLEANFAIL,
+                 states.DEPLOYFAIL, states.SERVICEFAIL], 'maintenance': True},
         predicate_extra_fields=['driver_internal_info'],
         predicate=lambda n: n.driver_internal_info.get('redfish_fw_updates'),
     )
@@ -313,7 +336,8 @@ class RedfishFirmware(base.FirmwareInterface):
     @periodics.node_periodic(
         purpose='checking async update of firmware component',
         spacing=CONF.redfish.firmware_update_fail_interval,
-        filters={'reserved': False, 'provision_state': states.CLEANWAIT},
+        filters={'reserved': False, 'provision_state_in': [states.CLEANWAIT,
+                 states.DEPLOYWAIT, states.SERVICEWAIT]},
         predicate_extra_fields=['driver_internal_info'],
         predicate=lambda n: n.driver_internal_info.get('redfish_fw_updates'),
     )
@@ -416,8 +440,10 @@ class RedfishFirmware(base.FirmwareInterface):
                 self._clear_updates(node)
                 if task.node.clean_step:
                     manager_utils.cleaning_error_handler(task, error_msg)
-                else:
+                elif task.node.deploy_step:
                     manager_utils.deploying_error_handler(task, error_msg)
+                elif task.node.service_step:
+                    manager_utils.servicing_error_handler(task, error_msg)
 
         else:
             LOG.debug('Firmware update in progress for node %(node)s, '
