@@ -15,6 +15,7 @@
 from oslo_config import cfg
 from oslo_log import log
 from oslo_utils import strutils
+from oslo_utils import timeutils
 from oslo_utils import uuidutils
 from oslo_utils import versionutils
 from oslo_versionedobjects import base as object_base
@@ -28,7 +29,7 @@ from ironic.objects import base
 from ironic.objects import fields as object_fields
 from ironic.objects import notification
 
-REQUIRED_INT_PROPERTIES = ['local_gb', 'cpus', 'memory_mb']
+REQUIRED_INT_PROPERTIES = ['local_gb', 'memory_mb']
 
 CONF = cfg.CONF
 LOG = log.getLogger(__name__)
@@ -76,7 +77,13 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
     # Version 1.33: Add retired and retired_reason fields
     # Version 1.34: Add lessee field
     # Version 1.35: Add network_data field
-    VERSION = '1.35'
+    # Version 1.36: Add boot_mode and secure_boot fields
+    # Version 1.37: Add shard field
+    # Version 1.38: Add parent_node field
+    # Version 1.39: Add firmware_interface field
+    # Version 1.40: Add service_step field
+    # Version 1.41: Add disable_power_off field
+    VERSION = '1.41'
 
     dbapi = db_api.get_instance()
 
@@ -102,6 +109,11 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
         # or has not yet started.
         'deploy_step': object_fields.FlexibleDictField(nullable=True),
 
+        # A service step dictionary, indicating the current step
+        # being executed, or None, indicating deployment is not in progress
+        # or has not yet started.
+        'service_step': object_fields.FlexibleDictField(nullable=True),
+
         'raid_config': object_fields.FlexibleDictField(nullable=True),
         'target_raid_config': object_fields.FlexibleDictField(nullable=True),
 
@@ -151,6 +163,7 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
         'boot_interface': object_fields.StringField(nullable=True),
         'console_interface': object_fields.StringField(nullable=True),
         'deploy_interface': object_fields.StringField(nullable=True),
+        'firmware_interface': object_fields.StringField(nullable=True),
         'inspect_interface': object_fields.StringField(nullable=True),
         'management_interface': object_fields.StringField(nullable=True),
         'network_interface': object_fields.StringField(nullable=True),
@@ -166,6 +179,11 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
         'retired': objects.fields.BooleanField(nullable=True),
         'retired_reason': object_fields.StringField(nullable=True),
         'network_data': object_fields.FlexibleDictField(nullable=True),
+        'boot_mode': object_fields.StringField(nullable=True),
+        'secure_boot': object_fields.BooleanField(nullable=True),
+        'shard': object_fields.StringField(nullable=True),
+        'parent_node': object_fields.StringField(nullable=True),
+        'disable_power_off': objects.fields.BooleanField(nullable=True),
     }
 
     def as_dict(self, secure=False, mask_configdrive=True):
@@ -174,17 +192,18 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
             d['driver_info'] = strutils.mask_dict_password(
                 d.get('driver_info', {}), "******")
             iinfo = d.pop('instance_info', {})
-            if not mask_configdrive:
-                configdrive = iinfo.pop('configdrive', None)
+            configdrive = iinfo.pop('configdrive', None)
             d['instance_info'] = strutils.mask_dict_password(iinfo, "******")
-            if not mask_configdrive and configdrive:
-                d['instance_info']['configdrive'] = configdrive
+            if configdrive is not None:
+                d['instance_info']['configdrive'] = (
+                    "******" if mask_configdrive else configdrive
+                )
             d['driver_internal_info'] = strutils.mask_dict_password(
                 d.get('driver_internal_info', {}), "******")
         return d
 
     def _validate_property_values(self, properties):
-        """Check if the input of local_gb, cpus and memory_mb are valid.
+        """Check if the input of local_gb and memory_mb are valid.
 
         :param properties: a dict contains the node's information.
         """
@@ -540,6 +559,12 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
         node = cls._from_db_object(context, cls(), db_node)
         return node
 
+    def get_interface(self, iface):
+        iface_name = '%s_interface' % iface
+        impl_name = self.instance_info.get(iface_name,
+                                           getattr(self, iface_name))
+        return impl_name
+
     def _convert_deploy_step_field(self, target_version,
                                    remove_unavailable_fields=True):
         # NOTE(rloo): Typically we set the value to None. However,
@@ -643,6 +668,13 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
             be set to None or removed.
         Version 1.35: network_data was added. For versions prior to this, it
             should be set to empty dict (or removed).
+        Version 1.36: boot_mode, secure_boot were was added. Defaults are None.
+            For versions prior to this, it should be set to None or removed.
+        Version 1.37: shard was added. Default is None. For versions prior to
+            this, it should be set to None or removed.
+        Version 1.39: firmware_interface field was added. Its default value is
+            None. For versions prior to this, it should be set to None (or
+            removed).
 
         :param target_version: the desired version of the object
         :param remove_unavailable_fields: True to remove fields that are
@@ -657,27 +689,101 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
                   ('bios_interface', 24), ('fault', 25),
                   ('automated_clean', 28), ('protected_reason', 29),
                   ('owner', 30), ('allocation_id', 31), ('description', 32),
-                  ('retired_reason', 33), ('lessee', 34)]
+                  ('retired_reason', 33), ('lessee', 34), ('boot_mode', 36),
+                  ('secure_boot', 36), ('shard', 37),
+                  ('firmware_interface', 39)]
 
         for name, minor in fields:
             self._adjust_field_to_version(name, None, target_version,
                                           1, minor, remove_unavailable_fields)
 
-        # NOTE(dtantsur): the default is False for protected
-        self._adjust_field_to_version('protected', False, target_version,
-                                      1, 29, remove_unavailable_fields)
+        boolean_fields = [('protected', 29, False),
+                          ('retired', 33, False),
+                          ('disable_power_off', 41, False)]
+
+        for name, minor, default in boolean_fields:
+            self._adjust_field_to_version(name, default, target_version,
+                                          1, minor, remove_unavailable_fields)
 
         self._convert_deploy_step_field(target_version,
                                         remove_unavailable_fields)
         self._convert_conductor_group_field(target_version,
                                             remove_unavailable_fields)
-
-        self._adjust_field_to_version('retired', False, target_version,
-                                      1, 33, remove_unavailable_fields)
-
         self._convert_network_data_field(target_version,
                                          remove_unavailable_fields)
 
+    def set_driver_internal_info(self, key, value):
+        """Set a `driver_internal_info` value.
+
+        Setting a `driver_internal_info` dict value via this method
+        ensures that this field will be flagged for saving.
+
+        :param key: Key of item to set
+        :param value: Value of item to set
+        """
+        self.driver_internal_info[key] = value
+        self._changed_fields.add('driver_internal_info')
+
+    def del_driver_internal_info(self, key, default_value=None):
+        """Pop a value from the driver_internal_info.
+
+        Removing a `driver_internal_info` dict value via this method
+        ensures that this field will be flagged for saving.
+
+        :param key: Key of item to pop off the `driver_internal_info` dict
+        :param default_value: Value to return if the key doesn't exist
+        :returns: The removed value, or `default_value`
+        """
+        if key in self.driver_internal_info:
+            self._changed_fields.add('driver_internal_info')
+            return self.driver_internal_info.pop(key, default_value)
+        return default_value
+
+    def timestamp_driver_internal_info(self, key):
+        """Set a `driver_internal_info` value with the current timestamp.
+
+        Setting a `driver_internal_info` timestamp value via this method
+        ensures that this field will be flagged for saving.
+
+        :param key: Key of item to set the timestamp on
+        """
+        self.set_driver_internal_info(key, timeutils.utcnow().isoformat())
+
+    def set_instance_info(self, key, value):
+        """Set an `instance_info` value.
+
+        Setting a `instance_info` dict value via this method ensures that this
+        field will be flagged for saving.
+
+        :param key: Key of item to set
+        :param value: Value of item to set
+        """
+        self.instance_info[key] = value
+        self._changed_fields.add('instance_info')
+
+    def set_property(self, key, value):
+        """Set a `properties` value.
+
+        Setting a `properties` dict value via this method ensures that this
+        field will be flagged for saving.
+
+        :param key: Key of item to set
+        :param value: Value of item to set
+        """
+        self.properties[key] = value
+        self._changed_fields.add('properties')
+
+    def list_child_node_ids(self, exclude_dedicated_power=False):
+        """Returns a list of node IDs for child nodes, if any.
+
+        :param exclude_dedicated_power: Boolean, Default False, if the list
+            should exclude nodes which are independently powered.
+        :returns: A list of any node_id values discovered in the database.
+        """
+        return self.dbapi.get_child_node_ids_by_parent_uuid(
+            self.uuid,
+            exclude_dedicated_power=exclude_dedicated_power)
+
 
 @base.IronicObjectRegistry.register
 class NodePayload(notification.NotificationPayloadBase):
@@ -697,8 +803,11 @@ class NodePayload(notification.NotificationPayloadBase):
         'created_at': ('node', 'created_at'),
         'deploy_step': ('node', 'deploy_step'),
         'description': ('node', 'description'),
+        'disable_power_off': ('node', 'disable_power_off'),
         'driver': ('node', 'driver'),
         'extra': ('node', 'extra'),
+        'boot_mode': ('node', 'boot_mode'),
+        'secure_boot': ('node', 'secure_boot'),
         'inspection_finished_at': ('node', 'inspection_finished_at'),
         'inspection_started_at': ('node', 'inspection_started_at'),
         'instance_uuid': ('node', 'instance_uuid'),
@@ -753,7 +862,9 @@ class NodePayload(notification.NotificationPayloadBase):
     # Version 1.13: Add description field.
     # Version 1.14: Add retired and retired_reason fields exposed via API.
     # Version 1.15: Add node lessee field.
-    VERSION = '1.15'
+    # Version 1.16: Add boot_mode and secure_boot fields.
+    # Version 1.17: Add disable_power_off field.
+    VERSION = '1.17'
     fields = {
         'clean_step': object_fields.FlexibleDictField(nullable=True),
         'conductor_group': object_fields.StringField(nullable=True),
@@ -761,8 +872,11 @@ class NodePayload(notification.NotificationPayloadBase):
         'created_at': object_fields.DateTimeField(nullable=True),
         'deploy_step': object_fields.FlexibleDictField(nullable=True),
         'description': object_fields.StringField(nullable=True),
+        'disable_power_off': objects.fields.BooleanField(nullable=True),
         'driver': object_fields.StringField(nullable=True),
         'extra': object_fields.FlexibleDictField(nullable=True),
+        'boot_mode': object_fields.StringField(nullable=True),
+        'secure_boot': object_fields.BooleanField(nullable=True),
         'inspection_finished_at': object_fields.DateTimeField(nullable=True),
         'inspection_started_at': object_fields.DateTimeField(nullable=True),
         'instance_uuid': object_fields.UUIDField(nullable=True),
@@ -842,7 +956,9 @@ class NodeSetPowerStatePayload(NodePayload):
     # Version 1.13: Parent NodePayload version 1.13
     # Version 1.14: Parent NodePayload version 1.14
     # Version 1.15: Parent NodePayload version 1.15
-    VERSION = '1.15'
+    # Version 1.16: Parent NodePayload version 1.16
+    # Version 1.17: Parent NodePayload version 1.17
+    VERSION = '1.17'
 
     fields = {
         # "to_power" indicates the future target_power_state of the node. A
@@ -898,7 +1014,9 @@ class NodeCorrectedPowerStatePayload(NodePayload):
     # Version 1.13: Parent NodePayload version 1.13
     # Version 1.14: Parent NodePayload version 1.14
     # Version 1.15: Parent NodePayload version 1.15
-    VERSION = '1.15'
+    # Version 1.16: Parent NodePayload version 1.16
+    # Version 1.17: Parent NodePayload version 1.17
+    VERSION = '1.17'
 
     fields = {
         'from_power': object_fields.StringField(nullable=True)
@@ -939,7 +1057,9 @@ class NodeSetProvisionStatePayload(NodePayload):
     # Version 1.14: Parent NodePayload version 1.14
     # Version 1.15: Parent NodePayload version 1.15
     # Version 1.16: add driver_internal_info
-    VERSION = '1.16'
+    # Version 1.17: Parent NodePayload version 1.16
+    # Version 1.18: Parent NodePayload version 1.17
+    VERSION = '1.18'
 
     SCHEMA = dict(NodePayload.SCHEMA,
                   **{'instance_info': ('node', 'instance_info'),
@@ -988,7 +1108,9 @@ class NodeCRUDPayload(NodePayload):
     # Version 1.11: Parent NodePayload version 1.13
     # Version 1.12: Parent NodePayload version 1.14
     # Version 1.13: Parent NodePayload version 1.15
-    VERSION = '1.13'
+    # Version 1.14: Parent NodePayload version 1.16
+    # Version 1.15: Parent NodePayload version 1.17
+    VERSION = '1.15'
 
     SCHEMA = dict(NodePayload.SCHEMA,
                   **{'instance_info': ('node', 'instance_info'),
@@ -1000,7 +1122,7 @@ class NodeCRUDPayload(NodePayload):
         'driver_info': object_fields.FlexibleDictField(nullable=True)
     }
 
-    def __init__(self, node, chassis_uuid):
+    def __init__(self, node, chassis_uuid=None):
         super(NodeCRUDPayload, self).__init__(node, chassis_uuid=chassis_uuid)
 
 
