@@ -27,37 +27,6 @@ from ironic.objects import fields as object_fields
 from ironic.objects import notification
 
 
-def migrate_vif_port_id(context, max_count):
-    """Copy portgroup's VIF info from extra to internal_info.
-
-    :param context: The context.
-    :param max_count: The maximum number of objects to migrate. Must be
-                      >= 0. If zero, all the objects will be migrated.
-    :returns: A 2-tuple -- the total number of objects that need to be
-              migrated (at the beginning of this call) and the number
-              of migrated objects.
-    """
-    # TODO(rloo): remove this method in Stein, when we remove from dbsync.py
-
-    # NOTE(rloo): if we introduce newer portgroup versions in the same cycle,
-    # we could add those versions along with 1.4. This is only so we don't
-    # duplicate work; it isn't necessary.
-    db_objs = Portgroup.dbapi.get_not_versions('Portgroup', ['1.4'])
-    total = len(db_objs)
-    max_count = max_count or total
-    done = 0
-    for db_obj in db_objs:
-        # NOTE(rloo): this will indirectly invoke
-        #             Portgroup._convert_to_version() which does all the real
-        #             work
-        portgroup = Portgroup._from_db_object(context, Portgroup(), db_obj)
-        portgroup.save()
-        done += 1
-        if done == max_count:
-            break
-    return total, done
-
-
 @base.IronicObjectRegistry.register
 class Portgroup(base.IronicObject, object_base.VersionedObjectDictCompat):
     # Version 1.0: Initial version
@@ -67,7 +36,8 @@ class Portgroup(base.IronicObject, object_base.VersionedObjectDictCompat):
     # Version 1.4: Migrate/copy extra['vif_port_id'] to
     #              internal_info['tenant_vif_port_id'] (not an explicit db
     #              change)
-    VERSION = '1.4'
+    # Version 1.5: Add node_uuid field
+    VERSION = '1.5'
 
     dbapi = dbapi.get_instance()
 
@@ -76,6 +46,7 @@ class Portgroup(base.IronicObject, object_base.VersionedObjectDictCompat):
         'uuid': object_fields.UUIDField(nullable=True),
         'name': object_fields.StringField(nullable=True),
         'node_id': object_fields.IntegerField(nullable=True),
+        'node_uuid': object_fields.UUIDField(nullable=True),
         'address': object_fields.MACAddressField(nullable=True),
         'extra': object_fields.FlexibleDictField(nullable=True),
         'internal_info': object_fields.FlexibleDictField(nullable=True),
@@ -183,17 +154,19 @@ class Portgroup(base.IronicObject, object_base.VersionedObjectDictCompat):
     # Implications of calling new remote procedures should be thought through.
     # @object_base.remotable_classmethod
     @classmethod
-    def get_by_address(cls, context, address):
+    def get_by_address(cls, context, address, project=None):
         """Find portgroup by address and return a :class:`Portgroup` object.
 
         :param cls: the :class:`Portgroup`
         :param context: Security context
         :param address: The MAC address of a portgroup.
+        :param project: a node owner or lessee to match against.
         :returns: A :class:`Portgroup` object.
         :raises: PortgroupNotFound
 
         """
-        db_portgroup = cls.dbapi.get_portgroup_by_address(address)
+        db_portgroup = cls.dbapi.get_portgroup_by_address(address,
+                                                          project=project)
         portgroup = cls._from_db_object(context, cls(), db_portgroup)
         return portgroup
 
@@ -222,7 +195,7 @@ class Portgroup(base.IronicObject, object_base.VersionedObjectDictCompat):
     # @object_base.remotable_classmethod
     @classmethod
     def list(cls, context, limit=None, marker=None,
-             sort_key=None, sort_dir=None):
+             sort_key=None, sort_dir=None, project=None):
         """Return a list of Portgroup objects.
 
         :param cls: the :class:`Portgroup`
@@ -231,6 +204,7 @@ class Portgroup(base.IronicObject, object_base.VersionedObjectDictCompat):
         :param marker: Pagination marker for large data sets.
         :param sort_key: Column to sort results by.
         :param sort_dir: Direction to sort. "asc" or "desc".
+        :param project: a node owner or lessee to match against.
         :returns: A list of :class:`Portgroup` object.
         :raises: InvalidParameterValue
 
@@ -238,7 +212,8 @@ class Portgroup(base.IronicObject, object_base.VersionedObjectDictCompat):
         db_portgroups = cls.dbapi.get_portgroup_list(limit=limit,
                                                      marker=marker,
                                                      sort_key=sort_key,
-                                                     sort_dir=sort_dir)
+                                                     sort_dir=sort_dir,
+                                                     project=project)
         return cls._from_db_object_list(context, db_portgroups)
 
     # NOTE(xek): We don't want to enable RPC on this call just yet. Remotable
@@ -247,7 +222,7 @@ class Portgroup(base.IronicObject, object_base.VersionedObjectDictCompat):
     # @object_base.remotable_classmethod
     @classmethod
     def list_by_node_id(cls, context, node_id, limit=None, marker=None,
-                        sort_key=None, sort_dir=None):
+                        sort_key=None, sort_dir=None, project=None):
         """Return a list of Portgroup objects associated with a given node ID.
 
         :param cls: the :class:`Portgroup`
@@ -257,6 +232,7 @@ class Portgroup(base.IronicObject, object_base.VersionedObjectDictCompat):
         :param marker: Pagination marker for large data sets.
         :param sort_key: Column to sort results by.
         :param sort_dir: Direction to sort. "asc" or "desc".
+        :param project: a node owner or lessee to match against.
         :returns: A list of :class:`Portgroup` object.
         :raises: InvalidParameterValue
 
@@ -265,7 +241,8 @@ class Portgroup(base.IronicObject, object_base.VersionedObjectDictCompat):
                                                             limit=limit,
                                                             marker=marker,
                                                             sort_key=sort_key,
-                                                            sort_dir=sort_dir)
+                                                            sort_dir=sort_dir,
+                                                            project=project)
         return cls._from_db_object_list(context, db_portgroups)
 
     # NOTE(xek): We don't want to enable RPC on this call just yet. Remotable
@@ -286,6 +263,10 @@ class Portgroup(base.IronicObject, object_base.VersionedObjectDictCompat):
         """
         values = self.do_version_changes_for_db()
         db_portgroup = self.dbapi.create_portgroup(values)
+        # NOTE(hjensas): To avoid lazy load issue (DetachedInstanceError) in
+        # sqlalchemy, get new port the port from the DB to ensure the node_uuid
+        # via association_proxy relationship is loaded.
+        db_portgroup = self.dbapi.get_portgroup_by_id(db_portgroup['id'])
         self._from_db_object(self._context, self, db_portgroup)
 
     # NOTE(xek): We don't want to enable RPC on this call just yet. Remotable
@@ -304,7 +285,7 @@ class Portgroup(base.IronicObject, object_base.VersionedObjectDictCompat):
         :raises: PortgroupNotEmpty, PortgroupNotFound
 
         """
-        self.dbapi.destroy_portgroup(self.uuid)
+        self.dbapi.destroy_portgroup(self.id)
         self.obj_reset_changes()
 
     # NOTE(xek): We don't want to enable RPC on this call just yet. Remotable
