@@ -14,29 +14,24 @@ Tests for the API /portgroups/ methods.
 """
 
 import datetime
+from http import client as http_client
+from unittest import mock
+from urllib import parse as urlparse
 
-import mock
 from oslo_config import cfg
 from oslo_utils import timeutils
 from oslo_utils import uuidutils
-import six
-from six.moves import http_client
-from six.moves.urllib import parse as urlparse
 from testtools.matchers import HasLength
-from wsme import types as wtypes
 
 from ironic.api.controllers import base as api_base
 from ironic.api.controllers import v1 as api_v1
 from ironic.api.controllers.v1 import notification_utils
-from ironic.api.controllers.v1 import portgroup as api_portgroup
 from ironic.api.controllers.v1 import utils as api_utils
 from ironic.common import exception
 from ironic.common import states
-from ironic.common import utils as common_utils
 from ironic.conductor import rpcapi
 from ironic import objects
 from ironic.objects import fields as obj_fields
-from ironic.tests import base
 from ironic.tests.unit.api import base as test_api_base
 from ironic.tests.unit.api import utils as apiutils
 from ironic.tests.unit.objects import utils as obj_utils
@@ -52,15 +47,6 @@ def _rpcapi_update_portgroup(self, context, portgroup, topic):
     return portgroup
 
 
-class TestPortgroupObject(base.TestCase):
-
-    def test_portgroup_init(self):
-        portgroup_dict = apiutils.portgroup_post_data(node_id=None)
-        del portgroup_dict['extra']
-        portgroup = api_portgroup.Portgroup(**portgroup_dict)
-        self.assertEqual(wtypes.Unset, portgroup.extra)
-
-
 class TestListPortgroups(test_api_base.BaseApiTest):
     headers = {api_base.Version.string: str(api_v1.max_version())}
 
@@ -98,10 +84,11 @@ class TestListPortgroups(test_api_base.BaseApiTest):
         self.assertNotIn('node_id', data)
 
     def test_get_one_with_json(self):
+        headers = {api_base.Version.string: '1.90'}
         portgroup = obj_utils.create_test_portgroup(self.context,
                                                     node_id=self.node.id)
         data = self.get_json('/portgroups/%s.json' % portgroup.uuid,
-                             headers=self.headers)
+                             headers=headers)
         self.assertEqual(portgroup.uuid, data['uuid'])
 
     def test_get_one_with_json_in_name(self):
@@ -112,6 +99,23 @@ class TestListPortgroups(test_api_base.BaseApiTest):
                              headers=self.headers)
         self.assertEqual(portgroup.uuid, data['uuid'])
 
+    def test_get_one_with_double_json_in_name(self):
+        portgroup = obj_utils.create_test_portgroup(self.context,
+                                                    name='pg.json.json',
+                                                    node_id=self.node.id)
+        data = self.get_json('/portgroups/%s' % portgroup.uuid,
+                             headers=self.headers)
+        self.assertEqual(portgroup.uuid, data['uuid'])
+
+    def test_get_one_with_json_not_found(self):
+        portgroup = obj_utils.create_test_portgroup(self.context,
+                                                    name='pg',
+                                                    node_id=self.node.id)
+        data = self.get_json('/portgroups/%s.json' % portgroup.uuid,
+                             headers=self.headers,
+                             expect_errors=True)
+        self.assertEqual(http_client.NOT_FOUND, data.status_code)
+
     def test_get_one_with_suffix(self):
         portgroup = obj_utils.create_test_portgroup(self.context,
                                                     name='pg.1',
@@ -128,7 +132,7 @@ class TestListPortgroups(test_api_base.BaseApiTest):
             '/portgroups/%s?fields=%s' % (portgroup.uuid, fields),
             headers=self.headers)
         # We always append "links"
-        self.assertItemsEqual(['address', 'extra', 'links'], data)
+        self.assertCountEqual(['address', 'extra', 'links'], data)
 
     def test_get_one_mode_field_lower_api_version(self):
         portgroup = obj_utils.create_test_portgroup(self.context,
@@ -158,7 +162,7 @@ class TestListPortgroups(test_api_base.BaseApiTest):
         self.assertEqual(3, len(data['portgroups']))
         for portgroup in data['portgroups']:
             # We always append "links"
-            self.assertItemsEqual(['uuid', 'extra', 'links'], portgroup)
+            self.assertCountEqual(['uuid', 'extra', 'links'], portgroup)
 
     def test_get_collection_properties_field_lower_api_version(self):
         obj_utils.create_test_portgroup(self.context, node_id=self.node.id)
@@ -277,7 +281,7 @@ class TestListPortgroups(test_api_base.BaseApiTest):
         self.assertEqual(len(portgroups), len(data['portgroups']))
 
         uuids = [n['uuid'] for n in data['portgroups']]
-        six.assertCountEqual(self, portgroups, uuids)
+        self.assertCountEqual(portgroups, uuids)
 
     def test_links(self):
         uuid = uuidutils.generate_uuid()
@@ -289,9 +293,9 @@ class TestListPortgroups(test_api_base.BaseApiTest):
         self.assertIn('ports', data)
         self.assertEqual(2, len(data['links']))
         self.assertIn(uuid, data['links'][0]['href'])
-        for l in data['links']:
-            bookmark = l['rel'] == 'bookmark'
-            self.assertTrue(self.validate_link(l['href'], bookmark=bookmark,
+        for link in data['links']:
+            bookmark = link['rel'] == 'bookmark'
+            self.assertTrue(self.validate_link(link['href'], bookmark=bookmark,
                             headers=self.headers))
 
     def test_collection_links(self):
@@ -327,6 +331,46 @@ class TestListPortgroups(test_api_base.BaseApiTest):
         next_marker = data['portgroups'][-1]['uuid']
         self.assertIn(next_marker, data['next'])
 
+    def test_collection_links_custom_fields(self):
+        fields = 'address,uuid'
+        cfg.CONF.set_override('max_limit', 3, 'api')
+        for i in range(5):
+            obj_utils.create_test_portgroup(
+                self.context,
+                uuid=uuidutils.generate_uuid(),
+                node_id=self.node.id,
+                name='portgroup%s' % i,
+                address='52:54:00:cf:2d:3%s' % i)
+
+        data = self.get_json(
+            '/portgroups?fields=%s' % fields,
+            headers={api_base.Version.string: str(api_v1.max_version())})
+
+        self.assertEqual(3, len(data['portgroups']))
+        next_marker = data['portgroups'][-1]['uuid']
+        self.assertIn(next_marker, data['next'])
+        self.assertIn('fields', data['next'])
+
+    def test_get_collection_pagination_no_uuid(self):
+        fields = 'address'
+        limit = 2
+        portgroups = []
+        for id_ in range(3):
+            portgroup = obj_utils.create_test_portgroup(
+                self.context,
+                node_id=self.node.id,
+                uuid=uuidutils.generate_uuid(),
+                name='portgroup%s' % id_,
+                address='52:54:00:cf:2d:3%s' % id_)
+            portgroups.append(portgroup)
+
+        data = self.get_json(
+            '/portgroups?fields=%s&limit=%s' % (fields, limit),
+            headers=self.headers)
+
+        self.assertEqual(limit, len(data['portgroups']))
+        self.assertIn('marker=%s' % portgroups[limit - 1].uuid, data['next'])
+
     def test_ports_subresource(self):
         pg = obj_utils.create_test_portgroup(self.context,
                                              uuid=uuidutils.generate_uuid(),
@@ -400,8 +444,7 @@ class TestListPortgroups(test_api_base.BaseApiTest):
                                  expect_errors=True,
                                  headers={api_base.Version.string: '1.23'})
         self.assertEqual(http_client.NOT_FOUND, response.status_int)
-        self.assertIn('The resource could not be found.',
-                      response.json['error_message'])
+        self.assertIn('Not Found', response.json['error_message'])
 
     def test_ports_subresource_portgroup_not_found(self):
         non_existent_uuid = 'eeeeeeee-cccc-aaaa-bbbb-cccccccccccc'
@@ -512,7 +555,7 @@ class TestListPortgroups(test_api_base.BaseApiTest):
     def test_detail_sort_key_not_allowed(self):
         self._test_sort_key_not_allowed(detail=True)
 
-    @mock.patch.object(api_utils, 'get_rpc_node')
+    @mock.patch.object(api_utils, 'get_rpc_node', autospec=True)
     def test_get_all_by_node_name_ok(self, mock_get_rpc_node):
         # GET /v1/portgroups specifying node_name - success
         mock_get_rpc_node.return_value = self.node
@@ -521,6 +564,8 @@ class TestListPortgroups(test_api_base.BaseApiTest):
                 node_id = self.node.id
             else:
                 node_id = 100000 + i
+                obj_utils.create_test_node(self.context, id=node_id,
+                                           uuid=uuidutils.generate_uuid())
             obj_utils.create_test_portgroup(
                 self.context,
                 node_id=node_id,
@@ -531,7 +576,7 @@ class TestListPortgroups(test_api_base.BaseApiTest):
                              headers=self.headers)
         self.assertEqual(3, len(data['portgroups']))
 
-    @mock.patch.object(api_utils, 'get_rpc_node')
+    @mock.patch.object(api_utils, 'get_rpc_node', autospec=True)
     def test_get_all_by_node_uuid_ok(self, mock_get_rpc_node):
         mock_get_rpc_node.return_value = self.node
         obj_utils.create_test_portgroup(self.context, node_id=self.node.id)
@@ -540,7 +585,7 @@ class TestListPortgroups(test_api_base.BaseApiTest):
         mock_get_rpc_node.assert_called_once_with(self.node.uuid)
         self.assertEqual(1, len(data['portgroups']))
 
-    @mock.patch.object(api_utils, 'get_rpc_node')
+    @mock.patch.object(api_utils, 'get_rpc_node', autospec=True)
     def test_detail_by_node_name_ok(self, mock_get_rpc_node):
         # GET /v1/portgroups/detail specifying node_name - success
         mock_get_rpc_node.return_value = self.node
@@ -552,7 +597,7 @@ class TestListPortgroups(test_api_base.BaseApiTest):
         self.assertEqual(self.node.uuid, data['portgroups'][0]['node_uuid'])
 
 
-@mock.patch.object(rpcapi.ConductorAPI, 'update_portgroup')
+@mock.patch.object(rpcapi.ConductorAPI, 'update_portgroup', autospec=True)
 class TestPatch(test_api_base.BaseApiTest):
     headers = {api_base.Version.string: str(api_v1.max_version())}
 
@@ -563,12 +608,14 @@ class TestPatch(test_api_base.BaseApiTest):
                                                          name='pg.1',
                                                          node_id=self.node.id)
 
-        p = mock.patch.object(rpcapi.ConductorAPI, 'get_topic_for')
+        p = mock.patch.object(rpcapi.ConductorAPI, 'get_topic_for',
+                              autospec=True)
         self.mock_gtf = p.start()
         self.mock_gtf.return_value = 'test-topic'
         self.addCleanup(p.stop)
 
-    @mock.patch.object(notification_utils, '_emit_api_notification')
+    @mock.patch.object(notification_utils, '_emit_api_notification',
+                       autospec=True)
     def test_update_byid(self, mock_notify, mock_upd):
         extra = {'foo': 'bar'}
         mock_upd.return_value = self.portgroup
@@ -582,7 +629,7 @@ class TestPatch(test_api_base.BaseApiTest):
         self.assertEqual(http_client.OK, response.status_code)
         self.assertEqual(extra, response.json['extra'])
 
-        kargs = mock_upd.call_args[0][1]
+        kargs = mock_upd.call_args[0][2]
         self.assertEqual(extra, kargs.extra)
         mock_notify.assert_has_calls([mock.call(mock.ANY, mock.ANY, 'update',
                                       obj_fields.NotificationLevel.INFO,
@@ -608,13 +655,14 @@ class TestPatch(test_api_base.BaseApiTest):
 
     def test_update_byname_with_json(self, mock_upd):
         extra = {'foo': 'bar'}
+        headers = {api_base.Version.string: '1.90'}
         mock_upd.return_value = self.portgroup
         mock_upd.return_value.extra = extra
         response = self.patch_json('/portgroups/%s.json' % self.portgroup.name,
                                    [{'path': '/extra/foo',
                                      'value': 'bar',
                                      'op': 'add'}],
-                                   headers=self.headers)
+                                   headers=headers)
         self.assertEqual('application/json', response.content_type)
         self.assertEqual(http_client.OK, response.status_code)
         self.assertEqual(extra, response.json['extra'])
@@ -684,10 +732,11 @@ class TestPatch(test_api_base.BaseApiTest):
         self.assertEqual(address, response.json['address'])
         self.assertTrue(mock_upd.called)
 
-        kargs = mock_upd.call_args[0][1]
+        kargs = mock_upd.call_args[0][2]
         self.assertEqual(address, kargs.address)
 
-    @mock.patch.object(notification_utils, '_emit_api_notification')
+    @mock.patch.object(notification_utils, '_emit_api_notification',
+                       autospec=True)
     def test_replace_address_already_exist(self, mock_notify, mock_upd):
         address = 'aa:aa:aa:aa:aa:aa'
         mock_upd.side_effect = exception.MACAlreadyExists(mac=address)
@@ -702,7 +751,7 @@ class TestPatch(test_api_base.BaseApiTest):
         self.assertTrue(response.json['error_message'])
         self.assertTrue(mock_upd.called)
 
-        kargs = mock_upd.call_args[0][1]
+        kargs = mock_upd.call_args[0][2]
         self.assertEqual(address, kargs.address)
         mock_notify.assert_has_calls([mock.call(mock.ANY, mock.ANY, 'update',
                                       obj_fields.NotificationLevel.INFO,
@@ -798,7 +847,7 @@ class TestPatch(test_api_base.BaseApiTest):
         self.assertEqual('application/json', response.content_type)
         self.assertEqual(http_client.OK, response.status_code)
         self.assertEqual(extra, response.json['extra'])
-        kargs = mock_upd.call_args[0][1]
+        kargs = mock_upd.call_args[0][2]
         self.assertEqual(extra, kargs.extra)
 
     def test_remove_multi(self, mock_upd):
@@ -817,7 +866,7 @@ class TestPatch(test_api_base.BaseApiTest):
         self.assertEqual('application/json', response.content_type)
         self.assertEqual(http_client.OK, response.status_code)
         self.assertEqual(extra, response.json['extra'])
-        kargs = mock_upd.call_args[0][1]
+        kargs = mock_upd.call_args[0][2]
         self.assertEqual(extra, kargs.extra)
 
         # Removing the collection
@@ -829,7 +878,7 @@ class TestPatch(test_api_base.BaseApiTest):
         self.assertEqual('application/json', response.content_type)
         self.assertEqual(http_client.OK, response.status_code)
         self.assertEqual({}, response.json['extra'])
-        kargs = mock_upd.call_args[0][1]
+        kargs = mock_upd.call_args[0][2]
         self.assertEqual(extra, kargs.extra)
 
         # Assert nothing else was changed
@@ -872,7 +921,7 @@ class TestPatch(test_api_base.BaseApiTest):
         self.assertEqual(http_client.OK, response.status_code)
         self.assertEqual(address, response.json['address'])
         self.assertTrue(mock_upd.called)
-        kargs = mock_upd.call_args[0][1]
+        kargs = mock_upd.call_args[0][2]
         self.assertEqual(address, kargs.address)
 
     def test_add_root_non_existent(self, mock_upd):
@@ -901,7 +950,7 @@ class TestPatch(test_api_base.BaseApiTest):
         self.assertEqual('application/json', response.content_type)
         self.assertEqual(http_client.OK, response.status_code)
         self.assertEqual(extra, response.json['extra'])
-        kargs = mock_upd.call_args[0][1]
+        kargs = mock_upd.call_args[0][2]
         self.assertEqual(extra, kargs.extra)
 
     def test_remove_uuid(self, mock_upd):
@@ -939,7 +988,7 @@ class TestPatch(test_api_base.BaseApiTest):
         self.assertEqual('application/json', response.content_type)
         self.assertEqual(http_client.OK, response.status_code)
         self.assertEqual(address.lower(), response.json['address'])
-        kargs = mock_upd.call_args[0][1]
+        kargs = mock_upd.call_args[0][2]
         self.assertEqual(address.lower(), kargs.address)
 
     def test_update_portgroup_standalone_ports_supported(self, mock_upd):
@@ -969,6 +1018,7 @@ class TestPatch(test_api_base.BaseApiTest):
         self.assertFalse(mock_upd.called)
 
     def test_update_portgroup_internal_info_not_allowed(self, mock_upd):
+        mock_upd.return_value = self.portgroup
         response = self.patch_json('/portgroups/%s' % self.portgroup.uuid,
                                    [{'path': '/internal_info',
                                      'value': False,
@@ -1014,6 +1064,7 @@ class TestPatch(test_api_base.BaseApiTest):
             [{'path': '/properties/abc', 'op': 'add', 'value': 123}], mock_upd)
 
     def test_remove_mode_not_allowed(self, mock_upd):
+        mock_upd.return_value = self.portgroup
         response = self.patch_json('/portgroups/%s' % self.portgroup.uuid,
                                    [{'path': '/mode',
                                      'op': 'remove'}],
@@ -1052,140 +1103,10 @@ class TestPatch(test_api_base.BaseApiTest):
         self.assertEqual(http_client.OK, response.status_int)
         self.assertEqual(address.lower(), response.json['address'])
         self.assertTrue(mock_upd.called)
-        kargs = mock_upd.call_args[0][1]
+        kargs = mock_upd.call_args[0][2]
         self.assertEqual(address.lower(), kargs.address)
 
 
-@mock.patch.object(rpcapi.ConductorAPI, 'update_portgroup', autospec=True,
-                   side_effect=_rpcapi_update_portgroup)
-class TestPatchExtraVifPortId(test_api_base.BaseApiTest):
-    headers = {api_base.Version.string: str(api_v1.max_version())}
-
-    def setUp(self):
-        super(TestPatchExtraVifPortId, self).setUp()
-        self.node = obj_utils.create_test_node(self.context)
-        self.portgroup = obj_utils.create_test_portgroup(self.context,
-                                                         node_id=self.node.id)
-        p = mock.patch.object(rpcapi.ConductorAPI, 'get_topic_for')
-        self.mock_gtf = p.start()
-        self.mock_gtf.return_value = 'test-topic'
-        self.addCleanup(p.stop)
-
-    def _test_add_extra_vif_port_id(self, headers, mock_warn, mock_upd):
-        extra = {'vif_port_id': 'bar'}
-        response = self.patch_json(
-            '/portgroups/%s' % self.portgroup.uuid,
-            [{'path': '/extra/vif_port_id', 'value': 'foo', 'op': 'add'},
-             {'path': '/extra/vif_port_id', 'value': 'bar', 'op': 'add'}],
-            headers=headers)
-        self.assertEqual('application/json', response.content_type)
-        self.assertEqual(http_client.OK, response.status_code)
-        self.assertEqual(extra, response.json['extra'])
-        return response
-
-    @mock.patch.object(common_utils, 'warn_about_deprecated_extra_vif_port_id',
-                       autospec=True)
-    def test_add_extra_vif_port_id(self, mock_warn, mock_upd):
-        expected_intern_info = self.portgroup.internal_info
-        expected_intern_info.update({'tenant_vif_port_id': 'bar'})
-        headers = {api_base.Version.string: '1.27'}
-        response = self._test_add_extra_vif_port_id(headers, mock_warn,
-                                                    mock_upd)
-        self.assertEqual(expected_intern_info, response.json['internal_info'])
-        self.assertFalse(mock_warn.called)
-
-    @mock.patch.object(common_utils, 'warn_about_deprecated_extra_vif_port_id',
-                       autospec=True)
-    def test_add_extra_vif_port_id_deprecated(self, mock_warn, mock_upd):
-        expected_intern_info = self.portgroup.internal_info
-        expected_intern_info.update({'tenant_vif_port_id': 'bar'})
-        response = self._test_add_extra_vif_port_id(self.headers, mock_warn,
-                                                    mock_upd)
-        self.assertEqual(expected_intern_info, response.json['internal_info'])
-        self.assertTrue(mock_warn.called)
-
-    @mock.patch.object(common_utils, 'warn_about_deprecated_extra_vif_port_id',
-                       autospec=True)
-    def test_replace_extra_vif_port_id(self, mock_warn, mock_upd):
-        self.portgroup.extra = {'vif_port_id': 'original'}
-        self.portgroup.internal_info = {'tenant_vif_port_id': 'original'}
-        self.portgroup.save()
-        expected_intern_info = self.portgroup.internal_info
-        expected_intern_info.update({'tenant_vif_port_id': 'bar'})
-        headers = {api_base.Version.string: '1.27'}
-        response = self._test_add_extra_vif_port_id(headers, mock_warn,
-                                                    mock_upd)
-        self.assertEqual(expected_intern_info, response.json['internal_info'])
-        self.assertFalse(mock_warn.called)
-
-    @mock.patch.object(common_utils, 'warn_about_deprecated_extra_vif_port_id',
-                       autospec=True)
-    def test_add_extra_vif_port_id_diff_internal(self, mock_warn, mock_upd):
-        internal_info = {'tenant_vif_port_id': 'original'}
-        self.portgroup.internal_info = internal_info
-        self.portgroup.save()
-        headers = {api_base.Version.string: '1.27'}
-        response = self._test_add_extra_vif_port_id(headers, mock_warn,
-                                                    mock_upd)
-        # not changed
-        self.assertEqual(internal_info, response.json['internal_info'])
-        self.assertFalse(mock_warn.called)
-
-    def _test_remove_extra_vif_port_id(self, headers, mock_warn, mock_upd):
-        response = self.patch_json(
-            '/portgroups/%s' % self.portgroup.uuid,
-            [{'path': '/extra/vif_port_id', 'op': 'remove'}],
-            headers=headers)
-        self.assertEqual('application/json', response.content_type)
-        self.assertEqual(http_client.OK, response.status_code)
-        self.assertEqual({}, response.json['extra'])
-        self.assertTrue(mock_upd.called)
-        return response
-
-    @mock.patch.object(common_utils, 'warn_about_deprecated_extra_vif_port_id',
-                       autospec=True)
-    def test_remove_extra_vif_port_id(self, mock_warn, mock_upd):
-        self.portgroup.extra = {'vif_port_id': 'bar'}
-        orig_info = self.portgroup.internal_info.copy()
-        intern_info = self.portgroup.internal_info
-        intern_info.update({'tenant_vif_port_id': 'bar'})
-        self.portgroup.internal_info = intern_info
-        self.portgroup.save()
-        headers = {api_base.Version.string: '1.27'}
-        response = self._test_remove_extra_vif_port_id(headers, mock_warn,
-                                                       mock_upd)
-        self.assertEqual(orig_info, response.json['internal_info'])
-        self.assertFalse(mock_warn.called)
-
-    @mock.patch.object(common_utils, 'warn_about_deprecated_extra_vif_port_id',
-                       autospec=True)
-    def test_remove_extra_vif_port_id_not_same(self, mock_warn, mock_upd):
-        # .internal_info['tenant_vif_port_id'] != .extra['vif_port_id']
-        self.portgroup.extra = {'vif_port_id': 'foo'}
-        intern_info = self.portgroup.internal_info
-        intern_info.update({'tenant_vif_port_id': 'bar'})
-        self.portgroup.internal_info = intern_info
-        self.portgroup.save()
-        headers = {api_base.Version.string: '1.28'}
-        response = self._test_remove_extra_vif_port_id(headers, mock_warn,
-                                                       mock_upd)
-        self.assertEqual(intern_info, response.json['internal_info'])
-        self.assertTrue(mock_warn.called)
-
-    @mock.patch.object(common_utils, 'warn_about_deprecated_extra_vif_port_id',
-                       autospec=True)
-    def test_remove_extra_vif_port_id_not_internal(self, mock_warn, mock_upd):
-        # no portgroup.internal_info['tenant_vif_port_id']
-        self.portgroup.extra = {'vif_port_id': 'foo'}
-        self.portgroup.save()
-        intern_info = self.portgroup.internal_info
-        headers = {api_base.Version.string: '1.28'}
-        response = self._test_remove_extra_vif_port_id(headers, mock_warn,
-                                                       mock_upd)
-        self.assertEqual(intern_info, response.json['internal_info'])
-        self.assertTrue(mock_warn.called)
-
-
 class TestPost(test_api_base.BaseApiTest):
     headers = {api_base.Version.string: str(api_v1.max_version())}
 
@@ -1193,11 +1114,10 @@ class TestPost(test_api_base.BaseApiTest):
         super(TestPost, self).setUp()
         self.node = obj_utils.create_test_node(self.context)
 
-    @mock.patch.object(notification_utils, '_emit_api_notification')
-    @mock.patch.object(common_utils, 'warn_about_deprecated_extra_vif_port_id',
+    @mock.patch.object(notification_utils, '_emit_api_notification',
                        autospec=True)
     @mock.patch.object(timeutils, 'utcnow', autospec=True)
-    def test_create_portgroup(self, mock_utcnow, mock_warn, mock_notify):
+    def test_create_portgroup(self, mock_utcnow, mock_notify):
         pdict = apiutils.post_get_test_portgroup()
         test_time = datetime.datetime(2000, 1, 1, 0, 0)
         mock_utcnow.return_value = test_time
@@ -1216,7 +1136,6 @@ class TestPost(test_api_base.BaseApiTest):
         expected_location = '/v1/portgroups/%s' % pdict['uuid']
         self.assertEqual(urlparse.urlparse(response.location).path,
                          expected_location)
-        self.assertEqual(0, mock_warn.call_count)
         mock_notify.assert_has_calls([mock.call(mock.ANY, mock.ANY, 'create',
                                       obj_fields.NotificationLevel.INFO,
                                       obj_fields.NotificationStatus.START,
@@ -1281,8 +1200,9 @@ class TestPost(test_api_base.BaseApiTest):
         self.assertFalse(mock_warn.called)
         self.assertFalse(mock_except.called)
 
-    @mock.patch.object(notification_utils, '_emit_api_notification')
-    @mock.patch.object(objects.Portgroup, 'create')
+    @mock.patch.object(notification_utils, '_emit_api_notification',
+                       autospec=True)
+    @mock.patch.object(objects.Portgroup, 'create', autospec=True)
     def test_create_portgroup_error(self, mock_create, mock_notify):
         mock_create.side_effect = Exception()
         pdict = apiutils.post_get_test_portgroup()
@@ -1306,41 +1226,6 @@ class TestPost(test_api_base.BaseApiTest):
                                headers=self.headers)
         self.assertEqual(pdict['extra'], result['extra'])
 
-    def _test_create_portgroup_with_extra_vif_port_id(self, headers,
-                                                      mock_warn):
-        pgdict = apiutils.post_get_test_portgroup(extra={'vif_port_id': 'foo'})
-        response = self.post_json('/portgroups', pgdict, headers=headers)
-        self.assertEqual('application/json', response.content_type)
-        self.assertEqual(http_client.CREATED, response.status_int)
-        self.assertEqual({'vif_port_id': 'foo'}, response.json['extra'])
-        self.assertEqual({'tenant_vif_port_id': 'foo'},
-                         response.json['internal_info'])
-
-    @mock.patch.object(common_utils, 'warn_about_deprecated_extra_vif_port_id',
-                       autospec=True)
-    def test_create_portgroup_with_extra_vif_port_id(self, mock_warn):
-        headers = {api_base.Version.string: '1.27'}
-        self._test_create_portgroup_with_extra_vif_port_id(headers, mock_warn)
-        self.assertFalse(mock_warn.called)
-
-    @mock.patch.object(common_utils, 'warn_about_deprecated_extra_vif_port_id',
-                       autospec=True)
-    def test_create_portgroup_with_extra_vif_port_id_deprecated(
-            self, mock_warn):
-        self._test_create_portgroup_with_extra_vif_port_id(
-            self.headers, mock_warn)
-        self.assertTrue(mock_warn.called)
-
-    @mock.patch.object(common_utils, 'warn_about_deprecated_extra_vif_port_id',
-                       autospec=True)
-    def test_create_portgroup_with_no_extra(self, mock_warn):
-        pgdict = apiutils.post_get_test_portgroup()
-        del pgdict['extra']
-        response = self.post_json('/portgroups', pgdict, headers=self.headers)
-        self.assertEqual('application/json', response.content_type)
-        self.assertEqual(http_client.CREATED, response.status_int)
-        self.assertEqual(0, mock_warn.call_count)
-
     def test_create_portgroup_no_address(self):
         pdict = apiutils.post_get_test_portgroup()
         del pdict['address']
@@ -1476,7 +1361,7 @@ class TestPost(test_api_base.BaseApiTest):
         self.assertEqual('active-backup', portgroup.mode)
 
 
-@mock.patch.object(rpcapi.ConductorAPI, 'destroy_portgroup')
+@mock.patch.object(rpcapi.ConductorAPI, 'destroy_portgroup', autospec=True)
 class TestDelete(test_api_base.BaseApiTest):
     headers = {api_base.Version.string: str(api_v1.max_version())}
 
@@ -1487,7 +1372,8 @@ class TestDelete(test_api_base.BaseApiTest):
                                                          name='pg.1',
                                                          node_id=self.node.id)
 
-        gtf = mock.patch.object(rpcapi.ConductorAPI, 'get_topic_for')
+        gtf = mock.patch.object(rpcapi.ConductorAPI, 'get_topic_for',
+                                autospec=True)
         self.mock_gtf = gtf.start()
         self.mock_gtf.return_value = 'test-topic'
         self.addCleanup(gtf.stop)
@@ -1499,7 +1385,8 @@ class TestDelete(test_api_base.BaseApiTest):
         self.assertEqual('application/json', response.content_type)
         self.assertIn(self.portgroup.address, response.json['error_message'])
 
-    @mock.patch.object(notification_utils, '_emit_api_notification')
+    @mock.patch.object(notification_utils, '_emit_api_notification',
+                       autospec=True)
     def test_delete_portgroup_byid(self, mock_notify, mock_dpt):
         self.delete('/portgroups/%s' % self.portgroup.uuid,
                     headers=self.headers)
@@ -1513,7 +1400,8 @@ class TestDelete(test_api_base.BaseApiTest):
                                       obj_fields.NotificationStatus.END,
                                       node_uuid=self.node.uuid)])
 
-    @mock.patch.object(notification_utils, '_emit_api_notification')
+    @mock.patch.object(notification_utils, '_emit_api_notification',
+                       autospec=True)
     def test_delete_portgroup_node_locked(self, mock_notify, mock_dpt):
         self.node.reserve(self.context, 'fake', self.node.uuid)
         mock_dpt.side_effect = exception.NodeLocked(node='fake-node',
@@ -1544,8 +1432,9 @@ class TestDelete(test_api_base.BaseApiTest):
         self.assertTrue(mock_dpt.called)
 
     def test_delete_portgroup_byname_with_json(self, mock_dpt):
+        headers = {api_base.Version.string: '1.90'}
         self.delete('/portgroups/%s.json' % self.portgroup.name,
-                    headers=self.headers)
+                    headers=headers)
         self.assertTrue(mock_dpt.called)
 
     def test_delete_portgroup_byname_not_existed(self, mock_dpt):
