@@ -44,7 +44,8 @@ NUMERIC_RELEASES = sorted(
     map(versionutils.convert_version_to_tuple,
         set(release_mappings.RELEASE_MAPPING)
         # Update the exceptions whenever needed
-        - {'master', 'zed', 'yoga'}),
+        - {'master', '2025.1', '2024.2', '2024.1', '2023.2', '2023.1',
+           'antelope', 'zed', 'yoga'}),
     reverse=True)
 
 
@@ -91,13 +92,17 @@ class ReleaseMappingsTestCase(base.TestCase):
 
     def test_contains_all_db_objects(self):
         self.assertIn('master', release_mappings.RELEASE_MAPPING)
-        model_names = set((s.__name__ for s in models.Base.__subclasses__()))
+        use_models = models.Base.__subclasses__()
+        use_models.append(models.Node)
+        model_names = set((s.__name__ for s in use_models))
         # NOTE(xek): As a rule, all models which can be changed between
         # releases or are sent through RPC should have their counterpart
         # versioned objects. Do not add an exception for such objects,
         # initialize them with the version 1.0 instead.
+        # NodeBase is also excluded as it is covered by Node.
         exceptions = set(['NodeTag', 'ConductorHardwareInterfaces',
-                          'NodeTrait', 'DeployTemplateStep'])
+                          'NodeTrait', 'DeployTemplateStep',
+                          'NodeBase', 'RunbookStep'])
         model_names -= exceptions
         # NodeTrait maps to two objects
         model_names |= set(['Trait', 'TraitList'])
@@ -117,13 +122,6 @@ class ReleaseMappingsTestCase(base.TestCase):
                     self.assertTrue(_check_versions_compatibility(
                         ver, registered_objects[obj_name][0].VERSION))
 
-    def test_no_gaps_in_release_versions(self):
-        for i, ver in enumerate(NUMERIC_RELEASES[:-1]):
-            prev = NUMERIC_RELEASES[i + 1]
-            if ver != (prev[0] + 1, 0) and ver != (prev[0], prev[1] + 1):
-                self.fail("Versions %s and %s are not sequential"
-                          % (prev, ver))
-
     def test_no_gaps_in_object_versions(self):
         oldest_release = '%d.%d' % NUMERIC_RELEASES[-1]
         oldest_versions = release_mappings.RELEASE_MAPPING[
