@@ -26,10 +26,13 @@ import oslo_messaging as messaging
 from oslo_messaging import _utils as messaging_utils
 
 from ironic.common import boot_devices
+from ironic.common import boot_modes
 from ironic.common import components
 from ironic.common import exception
 from ironic.common import indicator_states
+from ironic.common.json_rpc import client as json_rpc
 from ironic.common import release_mappings
+from ironic.common import rpc
 from ironic.common import states
 from ironic.conductor import manager as conductor_manager
 from ironic.conductor import rpcapi as conductor_rpcapi
@@ -74,6 +77,12 @@ class RPCAPITestCase(db_base.DbTestCase):
             self.context, objects.Node(), self.fake_node)
         self.fake_portgroup = db_utils.get_test_portgroup()
 
+    def test_rpc_disabled(self):
+        CONF.set_override('rpc_transport', 'none')
+        rpcapi = conductor_rpcapi.ConductorAPI(topic='fake-topic')
+        self.assertIsNone(rpcapi.client)
+        self.assertTrue(rpcapi._can_send_version('9.99'))
+
     def test_serialized_instance_has_uuid(self):
         self.assertIn('uuid', self.fake_node)
 
@@ -82,7 +91,12 @@ class RPCAPITestCase(db_base.DbTestCase):
         c = self.dbapi.register_conductor({'hostname': 'fake-host',
                                            'drivers': []})
         self.dbapi.register_conductor_hardware_interfaces(
-            c.id, 'fake-driver', 'deploy', ['iscsi', 'direct'], 'iscsi')
+            c.id,
+            [{'hardware_type': 'fake-driver', 'interface_type': 'deploy',
+              'interface_name': 'ansible', 'default': True},
+             {'hardware_type': 'fake-driver', 'interface_type': 'deploy',
+              'interface_name': 'direct', 'default': False}]
+        )
 
         rpcapi = conductor_rpcapi.ConductorAPI(topic='fake-topic')
         expected_topic = 'fake-topic.fake-host'
@@ -94,7 +108,12 @@ class RPCAPITestCase(db_base.DbTestCase):
         c = self.dbapi.register_conductor({'hostname': 'fake-host',
                                            'drivers': []})
         self.dbapi.register_conductor_hardware_interfaces(
-            c.id, 'other-driver', 'deploy', ['iscsi', 'direct'], 'iscsi')
+            c.id,
+            [{'hardware_type': 'other-driver', 'interface_type': 'deploy',
+              'interface_name': 'ansible', 'default': True},
+             {'hardware_type': 'other-driver', 'interface_type': 'deploy',
+              'interface_name': 'direct', 'default': False}]
+        )
 
         rpcapi = conductor_rpcapi.ConductorAPI(topic='fake-topic')
         self.assertRaises(exception.NoValidHost,
@@ -112,7 +131,12 @@ class RPCAPITestCase(db_base.DbTestCase):
         c = self.dbapi.register_conductor({'hostname': 'fake-host',
                                            'drivers': []})
         self.dbapi.register_conductor_hardware_interfaces(
-            c.id, 'fake-driver', 'deploy', ['iscsi', 'direct'], 'iscsi')
+            c.id,
+            [{'hardware_type': 'fake-driver', 'interface_type': 'deploy',
+              'interface_name': 'ansible', 'default': True},
+             {'hardware_type': 'fake-driver', 'interface_type': 'deploy',
+              'interface_name': 'direct', 'default': False}]
+        )
 
         rpcapi = conductor_rpcapi.ConductorAPI(topic='fake-topic')
         expected_topic = 'fake-topic.fake-host'
@@ -126,7 +150,12 @@ class RPCAPITestCase(db_base.DbTestCase):
             'drivers': [],
         })
         self.dbapi.register_conductor_hardware_interfaces(
-            c.id, 'fake-driver', 'deploy', ['iscsi', 'direct'], 'iscsi')
+            c.id,
+            [{'hardware_type': 'fake-driver', 'interface_type': 'deploy',
+              'interface_name': 'ansible', 'default': True},
+             {'hardware_type': 'fake-driver', 'interface_type': 'deploy',
+              'interface_name': 'direct', 'default': False}]
+        )
         rpcapi = conductor_rpcapi.ConductorAPI(topic='fake-topic')
         self.assertEqual('fake-topic.fake-host',
                          rpcapi.get_topic_for_driver('fake-driver'))
@@ -138,7 +167,12 @@ class RPCAPITestCase(db_base.DbTestCase):
             'drivers': [],
         })
         self.dbapi.register_conductor_hardware_interfaces(
-            c.id, 'fake-driver', 'deploy', ['iscsi', 'direct'], 'iscsi')
+            c.id,
+            [{'hardware_type': 'fake-driver', 'interface_type': 'deploy',
+              'interface_name': 'ansible', 'default': True},
+             {'hardware_type': 'fake-driver', 'interface_type': 'deploy',
+              'interface_name': 'direct', 'default': False}]
+        )
         rpcapi = conductor_rpcapi.ConductorAPI(topic='fake-topic')
         self.assertRaises(exception.DriverNotFound,
                           rpcapi.get_topic_for_driver,
@@ -156,7 +190,12 @@ class RPCAPITestCase(db_base.DbTestCase):
             'drivers': [],
         })
         self.dbapi.register_conductor_hardware_interfaces(
-            c.id, 'fake-driver', 'deploy', ['iscsi', 'direct'], 'iscsi')
+            c.id,
+            [{'hardware_type': 'fake-driver', 'interface_type': 'deploy',
+              'interface_name': 'ansible', 'default': True},
+             {'hardware_type': 'fake-driver', 'interface_type': 'deploy',
+              'interface_name': 'direct', 'default': False}]
+        )
         rpcapi = conductor_rpcapi.ConductorAPI(topic='fake-topic')
         self.assertEqual('fake-topic.fake-host',
                          rpcapi.get_topic_for_driver('fake-driver'))
@@ -166,7 +205,12 @@ class RPCAPITestCase(db_base.DbTestCase):
         c = self.dbapi.register_conductor({'hostname': 'fake-host',
                                            'drivers': []})
         self.dbapi.register_conductor_hardware_interfaces(
-            c.id, 'fake-driver', 'deploy', ['iscsi', 'direct'], 'iscsi')
+            c.id,
+            [{'hardware_type': 'fake-driver', 'interface_type': 'deploy',
+              'interface_name': 'ansible', 'default': True},
+             {'hardware_type': 'fake-driver', 'interface_type': 'deploy',
+              'interface_name': 'direct', 'default': False}]
+        )
         rpcapi = conductor_rpcapi.ConductorAPI()
         self.assertEqual(rpcapi.get_conductor_for(self.fake_node_obj),
                          'fake-host')
@@ -206,9 +250,7 @@ class RPCAPITestCase(db_base.DbTestCase):
 
         expected_retval = 'hello world' if rpc_method == 'call' else None
 
-        expected_topic = 'fake-topic'
-        if 'host' in kwargs:
-            expected_topic += ".%s" % kwargs['host']
+        expected_topic = kwargs.get('topic', 'fake-topic')
 
         target = {
             "topic": expected_topic,
@@ -266,6 +308,20 @@ class RPCAPITestCase(db_base.DbTestCase):
                           node_id=self.fake_node['uuid'],
                           new_state=states.POWER_ON)
 
+    def test_change_node_boot_mode(self):
+        self._test_rpcapi('change_node_boot_mode',
+                          'call',
+                          version='1.55',
+                          node_id=self.fake_node['uuid'],
+                          new_state=boot_modes.LEGACY_BIOS)
+
+    def test_change_node_secure_boot(self):
+        self._test_rpcapi('change_node_secure_boot',
+                          'call',
+                          version='1.55',
+                          node_id=self.fake_node['uuid'],
+                          new_state=True)
+
     def test_vendor_passthru(self):
         self._test_rpcapi('vendor_passthru',
                           'call',
@@ -292,6 +348,15 @@ class RPCAPITestCase(db_base.DbTestCase):
                           rebuild=False,
                           configdrive=None)
 
+    def test_do_node_deploy_with_deploy_steps(self):
+        self._test_rpcapi('do_node_deploy',
+                          'call',
+                          version='1.52',
+                          node_id=self.fake_node['uuid'],
+                          rebuild=False,
+                          configdrive=None,
+                          deploy_steps={'key': 'value'})
+
     def test_do_node_tear_down(self):
         self._test_rpcapi('do_node_tear_down',
                           'call',
@@ -422,6 +487,12 @@ class RPCAPITestCase(db_base.DbTestCase):
                           version='1.45',
                           node_id=self.fake_node['uuid'])
 
+    def test_continue_node_service(self):
+        self._test_rpcapi('continue_node_service',
+                          'cast',
+                          version='1.60',
+                          node_id=self.fake_node['uuid'])
+
     def test_get_raid_logical_disk_properties(self):
         self._test_rpcapi('get_raid_logical_disk_properties',
                           'call',
@@ -516,7 +587,7 @@ class RPCAPITestCase(db_base.DbTestCase):
                           node_id='fake-node',
                           callback_url='http://ramdisk.url:port',
                           agent_version=None,
-                          version='1.51')
+                          version='1.54')
 
     def test_heartbeat_agent_token(self):
         self._test_rpcapi('heartbeat',
@@ -525,7 +596,7 @@ class RPCAPITestCase(db_base.DbTestCase):
                           callback_url='http://ramdisk.url:port',
                           agent_version=None,
                           agent_token='xyz1',
-                          version='1.51')
+                          version='1.54')
 
     def test_destroy_volume_connector(self):
         fake_volume_connector = db_utils.get_test_volume_connector()
@@ -656,3 +727,107 @@ class RPCAPITestCase(db_base.DbTestCase):
                           'call',
                           allocation='fake-allocation',
                           version='1.48')
+
+    def test_do_node_service(self):
+        self._test_rpcapi('do_node_service',
+                          'call',
+                          node_id='fake-node',
+                          service_steps={'foo': 'bar'},
+                          disable_ramdisk=False,
+                          version='1.57')
+
+    @mock.patch.object(rpc, 'GLOBAL_MANAGER',
+                       spec_set=conductor_manager.ConductorManager)
+    def test_local_call(self, mock_manager):
+        CONF.set_override('host', 'fake.host')
+        rpcapi = conductor_rpcapi.ConductorAPI(topic='fake.topic')
+        rpcapi.create_node(mock.sentinel.context, mock.sentinel.node,
+                           topic='fake.topic.fake.host')
+        mock_manager.create_node.assert_called_once_with(
+            mock.sentinel.context, node_obj=mock.sentinel.node)
+
+    @mock.patch.object(rpc, 'GLOBAL_MANAGER',
+                       spec_set=conductor_manager.ConductorManager)
+    def test_local_call_with_rpc_disabled(self, mock_manager):
+        CONF.set_override('host', 'fake.host')
+        CONF.set_override('rpc_transport', 'none')
+        rpcapi = conductor_rpcapi.ConductorAPI(topic='fake.topic')
+        rpcapi.create_node(mock.sentinel.context, mock.sentinel.node,
+                           topic='fake.topic.fake.host')
+        mock_manager.create_node.assert_called_once_with(
+            mock.sentinel.context, node_obj=mock.sentinel.node)
+
+    @mock.patch.object(rpc, 'GLOBAL_MANAGER',
+                       spec_set=conductor_manager.ConductorManager)
+    def test_local_call_host_mismatch(self, mock_manager):
+        CONF.set_override('host', 'fake.host')
+        rpcapi = conductor_rpcapi.ConductorAPI(topic='fake.topic')
+        rpcapi.client = mock.Mock(spec_set=json_rpc.Client)
+        rpcapi.create_node(mock.sentinel.context, mock.sentinel.node,
+                           topic='fake.topic.not-fake.host')
+        mock_manager.create_node.assert_not_called()
+        rpcapi.client.prepare.assert_called_once_with(
+            topic='fake.topic.not-fake.host', version=mock.ANY)
+
+    @mock.patch.object(rpc, 'GLOBAL_MANAGER',
+                       spec_set=conductor_manager.ConductorManager)
+    def test_local_call_host_mismatch_with_rpc_disabled(self, mock_manager):
+        CONF.set_override('host', 'fake.host')
+        CONF.set_override('rpc_transport', 'none')
+        rpcapi = conductor_rpcapi.ConductorAPI(topic='fake.topic')
+        self.assertRaises(exception.ServiceUnavailable,
+                          rpcapi.create_node,
+                          mock.sentinel.context, mock.sentinel.node,
+                          topic='fake.topic.not-fake.host')
+
+    @mock.patch.object(rpc, 'GLOBAL_MANAGER', None)
+    def test_local_call_no_conductor_with_rpc_disabled(self):
+        CONF.set_override('host', 'fake.host')
+        CONF.set_override('rpc_transport', 'none')
+        rpcapi = conductor_rpcapi.ConductorAPI(topic='fake.topic')
+        self.assertRaises(exception.ServiceUnavailable,
+                          rpcapi.create_node,
+                          mock.sentinel.context, mock.sentinel.node,
+                          topic='fake.topic.fake.host')
+
+    @mock.patch.object(rpc, 'GLOBAL_MANAGER',
+                       spec_set=conductor_manager.ConductorManager)
+    def test_local_cast(self, mock_manager):
+        CONF.set_override('host', 'fake.host')
+        rpcapi = conductor_rpcapi.ConductorAPI(topic='fake.topic')
+        cctxt = rpcapi._prepare_call(topic='fake.topic.fake.host')
+        cctxt.cast(mock.sentinel.context, 'create_node',
+                   node_obj=mock.sentinel.node)
+        mock_manager.create_node.assert_called_once_with(
+            mock.sentinel.context, node_obj=mock.sentinel.node)
+
+    @mock.patch.object(conductor_rpcapi.LOG, 'exception', autospec=True)
+    @mock.patch.object(rpc, 'GLOBAL_MANAGER',
+                       spec_set=conductor_manager.ConductorManager)
+    def test_local_cast_error(self, mock_manager, mock_log):
+        CONF.set_override('host', 'fake.host')
+        mock_manager.create_node.side_effect = RuntimeError('boom')
+        rpcapi = conductor_rpcapi.ConductorAPI(topic='fake.topic')
+        cctxt = rpcapi._prepare_call(topic='fake.topic.fake.host')
+        cctxt.cast(mock.sentinel.context, 'create_node',
+                   node_obj=mock.sentinel.node)
+        mock_manager.create_node.assert_called_once_with(
+            mock.sentinel.context, node_obj=mock.sentinel.node)
+        self.assertTrue(mock_log.called)
+
+    @mock.patch.object(rpc, 'GLOBAL_MANAGER',
+                       spec_set=conductor_manager.ConductorManager)
+    def test_local_call_expected_exception(self, mock_manager):
+        @messaging.expected_exceptions(exception.InvalidParameterValue)
+        def fake_create(context, node_obj):
+            raise exception.InvalidParameterValue('sorry')
+
+        CONF.set_override('host', 'fake.host')
+        rpcapi = conductor_rpcapi.ConductorAPI(topic='fake.topic')
+        mock_manager.create_node.side_effect = fake_create
+        self.assertRaisesRegex(exception.InvalidParameterValue, 'sorry',
+                               rpcapi.create_node,
+                               mock.sentinel.context, mock.sentinel.node,
+                               topic='fake.topic.fake.host')
+        mock_manager.create_node.assert_called_once_with(
+            mock.sentinel.context, node_obj=mock.sentinel.node)
