@@ -13,7 +13,6 @@
 import json
 from unittest import mock
 
-from ironic_lib import utils as irlib_utils
 from oslo_concurrency import processutils
 
 from ironic.common import exception
@@ -242,6 +241,7 @@ class TestAnsibleMethods(AnsibleDeployTestCaseBase):
             '--private-key=/path/to/key')
 
     def test__parse_partitioning_info_root_msdos(self):
+        self.config(default_boot_mode='bios', group='deploy')
         expected_info = {
             'partition_info': {
                 'label': 'msdos',
@@ -495,6 +495,36 @@ class TestAnsibleMethods(AnsibleDeployTestCaseBase):
                 mock.call().write('fake-content'),
                 mock.call().__exit__(None, None, None)))
 
+    @mock.patch.object(utils, 'build_configdrive', autospec=True)
+    def test__prepare_variables_configdrive_json(self, mock_build_configdrive):
+        i_info = self.node.instance_info
+        i_info['configdrive'] = {'meta_data': {}}
+        self.node.instance_info = i_info
+        self.node.save()
+        mock_build_configdrive.return_value = 'fake-content'
+        configdrive_path = ('%(tempdir)s/%(node)s.cndrive' %
+                            {'tempdir': ansible_deploy.CONF.tempdir,
+                             'node': self.node.uuid})
+        expected = {"image": {"url": "http://image",
+                              "validate_certs": "yes",
+                              "source": "fake-image",
+                              "disk_format": "qcow2",
+                              "checksum": "md5:checksum"},
+                    'configdrive': {'type': 'file',
+                                    'location': configdrive_path}}
+        with mock.patch.object(ansible_deploy, 'open', mock.mock_open(),
+                               create=True) as open_mock:
+            with task_manager.acquire(self.context, self.node.uuid) as task:
+                self.assertEqual(expected,
+                                 ansible_deploy._prepare_variables(task))
+                mock_build_configdrive.assert_called_once_with(
+                    task.node, {'meta_data': {}})
+            open_mock.assert_has_calls((
+                mock.call(configdrive_path, 'w'),
+                mock.call().__enter__(),
+                mock.call().write('fake-content'),
+                mock.call().__exit__(None, None, None)))
+
     def test__validate_clean_steps(self):
         steps = [{"interface": "deploy",
                   "name": "foo",
@@ -592,24 +622,6 @@ class TestAnsibleDeploy(AnsibleDeployTestCaseBase):
                 {'instance_info.image_source': INSTANCE_INFO['image_source']},
                 mock.ANY)
 
-    @mock.patch.object(deploy_utils, 'get_boot_option',
-                       return_value='netboot', autospec=True)
-    @mock.patch.object(pxe.PXEBoot, 'validate', autospec=True)
-    def test_validate_not_iwdi_netboot(self, pxe_boot_validate_mock,
-                                       get_boot_mock):
-        driver_internal_info = dict(DRIVER_INTERNAL_INFO)
-        driver_internal_info['is_whole_disk_image'] = False
-        self.node.driver_internal_info = driver_internal_info
-        self.node.save()
-
-        with task_manager.acquire(
-                self.context, self.node['uuid'], shared=False) as task:
-            self.assertRaises(exception.InvalidParameterValue,
-                              self.driver.validate, task)
-            pxe_boot_validate_mock.assert_called_once_with(
-                task.driver.boot, task)
-            get_boot_mock.assert_called_once_with(task.node)
-
     @mock.patch.object(ansible_deploy, '_calculate_memory_req', autospec=True,
                        return_value=2000)
     @mock.patch.object(utils, 'node_power_action', autospec=True)
@@ -663,7 +675,7 @@ class TestAnsibleDeploy(AnsibleDeployTestCaseBase):
 
     @mock.patch.object(ansible_deploy, '_get_configdrive_path',
                        return_value='/path/test', autospec=True)
-    @mock.patch.object(irlib_utils, 'unlink_without_raise', autospec=True)
+    @mock.patch.object(com_utils, 'unlink_without_raise', autospec=True)
     @mock.patch.object(pxe.PXEBoot, 'clean_up_ramdisk', autospec=True)
     def test_clean_up(self, pxe_clean_up_mock, unlink_mock,
                       get_cfdrive_path_mock):
