@@ -85,26 +85,33 @@ class NeutronInterfaceTestCase(db_base.DbTestCase):
             self.interface.port_changed(task, port)
             mock_p_changed.assert_called_once_with(self.interface, task, port)
 
-    def test_init_incorrect_provisioning_net(self):
-        self.config(provisioning_network=None, group='neutron')
-        self.assertRaises(exception.DriverLoadError, neutron.NeutronNetwork)
-        self.config(provisioning_network=uuidutils.generate_uuid(),
-                    group='neutron')
-        self.config(cleaning_network=None, group='neutron')
-        self.assertRaises(exception.DriverLoadError, neutron.NeutronNetwork)
-
     @mock.patch.object(neutron_common, 'validate_network', autospec=True)
     def test_validate(self, validate_mock):
         with task_manager.acquire(self.context, self.node.id) as task:
             self.interface.validate(task)
+            # NOTE(TheJulia): This tests validates the calls are made.
+            # When not mocked out completely, since Neutron is consulted
+            # on validity of the name or UUID as well, the validate_network
+            # method gets called which raises a validate parsable error.
             self.assertEqual([mock.call(CONF.neutron.cleaning_network,
-                                        'cleaning network',
+                                        'cleaning_network',
                                         context=task.context),
                               mock.call(CONF.neutron.provisioning_network,
-                                        'provisioning network',
+                                        'provisioning_network',
                                         context=task.context)],
                              validate_mock.call_args_list)
 
+    @mock.patch.object(neutron_common, 'validate_network', autospec=True)
+    def test_validate_with_disable_power_off(self, validate_mock):
+        with task_manager.acquire(self.context, self.node.id) as task:
+            task.node.disable_power_off = True
+            self.assertRaises(exception.InvalidParameterValue,
+                              self.interface.validate, task)
+
+            CONF.set_override('allow_disabling_power_off', True,
+                              group='neutron')
+            self.interface.validate(task)
+
     @mock.patch.object(neutron_common, 'validate_network',
                        side_effect=lambda n, t, context=None: n, autospec=True)
     @mock.patch.object(neutron_common, 'rollback_ports', autospec=True)
@@ -123,7 +130,7 @@ class NeutronInterfaceTestCase(db_base.DbTestCase):
                 security_groups=[])
             validate_mock.assert_called_once_with(
                 CONF.neutron.provisioning_network,
-                'provisioning network', context=task.context)
+                'provisioning_network', context=task.context)
         self.port.refresh()
         self.assertEqual(self.neutron_port.id,
                          self.port.internal_info['provisioning_vif_port_id'])
@@ -155,7 +162,7 @@ class NeutronInterfaceTestCase(db_base.DbTestCase):
                     security_groups=[])
                 validate_mock.assert_called_once_with(
                     provisioning_network_uuid,
-                    'provisioning network', context=task.context)
+                    'provisioning_network', context=task.context)
         self.port.refresh()
         self.assertEqual(self.neutron_port.id,
                          self.port.internal_info['provisioning_vif_port_id'])
@@ -199,7 +206,7 @@ class NeutronInterfaceTestCase(db_base.DbTestCase):
                 task, CONF.neutron.provisioning_network)
             validate_mock.assert_called_once_with(
                 CONF.neutron.provisioning_network,
-                'provisioning network', context=task.context)
+                'provisioning_network', context=task.context)
         self.port.refresh()
         self.assertNotIn('provisioning_vif_port_id', self.port.internal_info)
 
@@ -222,7 +229,7 @@ class NeutronInterfaceTestCase(db_base.DbTestCase):
                 task, provisioning_network_uuid)
             validate_mock.assert_called_once_with(
                 provisioning_network_uuid,
-                'provisioning network', context=task.context)
+                'provisioning_network', context=task.context)
         self.port.refresh()
         self.assertNotIn('provisioning_vif_port_id', self.port.internal_info)
 
@@ -240,7 +247,7 @@ class NeutronInterfaceTestCase(db_base.DbTestCase):
             self.assertEqual(res, add_ports_mock.return_value)
             validate_mock.assert_called_once_with(
                 CONF.neutron.cleaning_network,
-                'cleaning network', context=task.context)
+                'cleaning_network', context=task.context)
         self.port.refresh()
         self.assertEqual(self.neutron_port.id,
                          self.port.internal_info['cleaning_vif_port_id'])
@@ -266,7 +273,7 @@ class NeutronInterfaceTestCase(db_base.DbTestCase):
                 self.assertEqual(res, add_ports_mock.return_value)
                 validate_mock.assert_called_once_with(
                     cleaning_network_uuid,
-                    'cleaning network', context=task.context)
+                    'cleaning_network', context=task.context)
         self.port.refresh()
         self.assertEqual(self.neutron_port.id,
                          self.port.internal_info['cleaning_vif_port_id'])
@@ -307,7 +314,7 @@ class NeutronInterfaceTestCase(db_base.DbTestCase):
                 task, CONF.neutron.cleaning_network)
             validate_mock.assert_called_once_with(
                 CONF.neutron.cleaning_network,
-                'cleaning network', context=task.context)
+                'cleaning_network', context=task.context)
         self.port.refresh()
         self.assertNotIn('cleaning_vif_port_id', self.port.internal_info)
 
@@ -330,7 +337,7 @@ class NeutronInterfaceTestCase(db_base.DbTestCase):
                 task, cleaning_network_uuid)
             validate_mock.assert_called_once_with(
                 cleaning_network_uuid,
-                'cleaning network', context=task.context)
+                'cleaning_network', context=task.context)
         self.port.refresh()
         self.assertNotIn('cleaning_vif_port_id', self.port.internal_info)
 
@@ -345,14 +352,14 @@ class NeutronInterfaceTestCase(db_base.DbTestCase):
         with task_manager.acquire(self.context, self.node.id) as task:
             self.interface.validate_rescue(task)
             validate_mock.assert_called_once_with(
-                rescuing_network_uuid, 'rescuing network',
+                rescuing_network_uuid, 'rescuing_network',
                 context=task.context),
 
     def test_validate_rescue_exc(self):
         self.config(rescuing_network="", group='neutron')
         with task_manager.acquire(self.context, self.node.id) as task:
             self.assertRaisesRegex(exception.MissingParameterValue,
-                                   'rescuing network is not set',
+                                   'rescuing_network is not set',
                                    self.interface.validate_rescue, task)
 
     @mock.patch.object(neutron_common, 'validate_network',
@@ -380,7 +387,7 @@ class NeutronInterfaceTestCase(db_base.DbTestCase):
             self.assertEqual(add_ports_mock.return_value, res)
             validate_mock.assert_called_once_with(
                 CONF.neutron.rescuing_network,
-                'rescuing network', context=task.context)
+                'rescuing_network', context=task.context)
         other_port.refresh()
         self.assertEqual(neutron_other_port['id'],
                          other_port.internal_info['rescuing_vif_port_id'])
@@ -416,7 +423,7 @@ class NeutronInterfaceTestCase(db_base.DbTestCase):
             self.assertEqual(add_ports_mock.return_value, res)
             validate_mock.assert_called_once_with(
                 rescuing_network_uuid,
-                'rescuing network', context=task.context)
+                'rescuing_network', context=task.context)
         other_port.refresh()
         self.assertEqual(neutron_other_port['id'],
                          other_port.internal_info['rescuing_vif_port_id'])
@@ -463,7 +470,7 @@ class NeutronInterfaceTestCase(db_base.DbTestCase):
                 task, CONF.neutron.rescuing_network)
             validate_mock.assert_called_once_with(
                 CONF.neutron.rescuing_network,
-                'rescuing network', context=task.context)
+                'rescuing_network', context=task.context)
         other_port.refresh()
         self.assertNotIn('rescuing_vif_port_id', self.port.internal_info)
         self.assertNotIn('rescuing_vif_port_id', other_port.internal_info)
@@ -803,7 +810,7 @@ class NeutronInterfaceTestCase(db_base.DbTestCase):
             self.assertEqual(res, add_ports_mock.return_value)
             validate_mock.assert_called_once_with(
                 CONF.neutron.inspection_network,
-                'inspection network', context=task.context)
+                'inspection_network', context=task.context)
         self.port.refresh()
         self.assertEqual(self.neutron_port.id,
                          self.port.internal_info['inspection_vif_port_id'])
@@ -830,7 +837,7 @@ class NeutronInterfaceTestCase(db_base.DbTestCase):
                 self.assertEqual(res, add_ports_mock.return_value)
                 validate_mock.assert_called_once_with(
                     inspection_network_uuid,
-                    'inspection network', context=task.context)
+                    'inspection_network', context=task.context)
         self.port.refresh()
         self.assertEqual(self.neutron_port.id,
                          self.port.internal_info['inspection_vif_port_id'])
@@ -870,7 +877,7 @@ class NeutronInterfaceTestCase(db_base.DbTestCase):
         with task_manager.acquire(self.context, self.node.id) as task:
             self.interface.validate_inspection(task)
             validate_mock.assert_called_once_with(
-                inspection_network_uuid, 'inspection network',
+                inspection_network_uuid, 'inspection_network',
                 context=task.context),
 
     def test_validate_inspection_exc(self):
