@@ -14,14 +14,13 @@
 
 from unittest import mock
 
-from oslo_utils import importutils
+import sushy
 
 from ironic.common import exception
 from ironic.common import states
 from ironic.conductor import task_manager
 from ironic.conductor import utils as manager_utils
 from ironic.drivers.modules import deploy_utils
-from ironic.drivers.modules.redfish import bios as redfish_bios
 from ironic.drivers.modules.redfish import boot as redfish_boot
 from ironic.drivers.modules.redfish import utils as redfish_utils
 from ironic import objects
@@ -29,8 +28,6 @@ from ironic.tests.unit.db import base as db_base
 from ironic.tests.unit.db import utils as db_utils
 from ironic.tests.unit.objects import utils as obj_utils
 
-sushy = importutils.try_import('sushy')
-
 INFO_DICT = db_utils.get_test_redfish_info()
 
 
@@ -57,13 +54,6 @@ class RedfishBiosTestCase(db_base.DbTestCase):
         self.node = obj_utils.create_test_node(
             self.context, driver='redfish', driver_info=INFO_DICT)
 
-    @mock.patch.object(redfish_bios, 'sushy', None)
-    def test_loading_error(self):
-        self.assertRaisesRegex(
-            exception.DriverLoadError,
-            'Unable to import the sushy library',
-            redfish_bios.RedfishBIOS)
-
     def test_get_properties(self):
         with task_manager.acquire(self.context, self.node.uuid,
                                   shared=True) as task:
@@ -99,7 +89,9 @@ class RedfishBiosTestCase(db_base.DbTestCase):
             task.driver.bios.cache_bios_settings(task)
             mock_get_system.assert_called_once_with(task.node)
             mock_setting_list.sync_node_setting.assert_called_once_with(
-                task.context, task.node.id, [{'name': 'foo', 'value': 'bar'}])
+                task.context, task.node.id,
+                [{'name': 'foo', 'value': 'bar'}])
+
             mock_setting_list.create.assert_not_called()
             mock_setting_list.save.assert_not_called()
             mock_setting_list.delete.assert_not_called()
@@ -153,7 +145,9 @@ class RedfishBiosTestCase(db_base.DbTestCase):
             task.driver.bios.cache_bios_settings(task)
             mock_get_system.assert_called_once_with(task.node)
             mock_setting_list.sync_node_setting.assert_called_once_with(
-                task.context, task.node.id, [{'name': 'foo', 'value': 'bar'}])
+                task.context, task.node.id,
+                [{'name': 'foo', 'value': 'bar'}])
+
             mock_setting_list.create.assert_called_once_with(
                 task.context, task.node.id, create_list)
             mock_setting_list.save.assert_called_once_with(
@@ -181,10 +175,8 @@ class RedfishBiosTestCase(db_base.DbTestCase):
             expected_ret = states.DEPLOYWAIT
         data = step_data['argsinfo'].get('settings', None)
         step = step_data['step']
-        if step == 'factory_reset':
-            check_fields.append('post_factory_reset_reboot_requested')
-        elif step == 'apply_configuration':
-            check_fields.append('post_config_reboot_requested')
+        check_fields.append('post_bios_reboot_requested')
+        if step == 'apply_configuration':
             attributes = {s['name']: s['value'] for s in data}
         mock_build_agent_options.return_value = {'a': 'b'}
         with task_manager.acquire(self.context, self.node.uuid,
@@ -199,10 +191,11 @@ class RedfishBiosTestCase(db_base.DbTestCase):
             if fast_track:
                 mock_power_action.assert_has_calls([
                     mock.call(task, states.POWER_OFF),
-                    mock.call(task, states.REBOOT),
+                    mock.call(task, states.REBOOT, None),
                 ])
             else:
-                mock_power_action.assert_called_once_with(task, states.REBOOT)
+                mock_power_action.assert_called_once_with(
+                    task, states.REBOOT, None)
             if step == 'factory_reset':
                 bios.reset_bios.assert_called_once()
             if step == 'apply_configuration':
@@ -268,19 +261,18 @@ class RedfishBiosTestCase(db_base.DbTestCase):
             step_data = self.node.clean_step
         data = step_data['argsinfo'].get('settings', None)
         step = step_data['step']
-        if step == 'factory_reset':
-            check_fields = ['post_factory_reset_reboot_requested']
+        check_fields = ['post_bios_reboot_requested']
         if step == 'apply_configuration':
             mock_bios = mock.Mock()
             # if attributes after reboot not provided then mimic success
             # by returning the same as requested
             mock_bios.attributes = attributes_after_reboot \
                 or self.node.driver_internal_info['requested_bios_attrs']
+            mock_bios.get_attribute_registry = []
             mock_system = mock.Mock()
             mock_system.bios = mock_bios
             mock_get_system.return_value = mock_system
-            check_fields = ['post_config_reboot_requested',
-                            'requested_bios_attrs']
+            check_fields.append('requested_bios_attrs')
         with task_manager.acquire(self.context, self.node.uuid,
                                   shared=False) as task:
             if step == 'factory_reset':
@@ -297,7 +289,7 @@ class RedfishBiosTestCase(db_base.DbTestCase):
                                 'step': 'factory_reset', 'argsinfo': {}}
         node = self.node
         driver_internal_info = node.driver_internal_info
-        driver_internal_info['post_factory_reset_reboot_requested'] = True
+        driver_internal_info['post_bios_reboot_requested'] = True
         node.driver_internal_info = driver_internal_info
         node.save()
         self._test_step_post_reboot()
@@ -307,7 +299,7 @@ class RedfishBiosTestCase(db_base.DbTestCase):
                                  'step': 'factory_reset', 'argsinfo': {}}
         node = self.node
         driver_internal_info = node.driver_internal_info
-        driver_internal_info['post_factory_reset_reboot_requested'] = True
+        driver_internal_info['post_bios_reboot_requested'] = True
         node.driver_internal_info = driver_internal_info
         node.save()
         self._test_step_post_reboot()
@@ -326,7 +318,7 @@ class RedfishBiosTestCase(db_base.DbTestCase):
                            'NicBoot1': 'NetworkBoot'}
         node = self.node
         driver_internal_info = node.driver_internal_info
-        driver_internal_info['post_config_reboot_requested'] = True
+        driver_internal_info['post_bios_reboot_requested'] = True
         driver_internal_info['requested_bios_attrs'] = requested_attrs
         self.node.driver_internal_info = driver_internal_info
         self.node.save()
@@ -337,7 +329,8 @@ class RedfishBiosTestCase(db_base.DbTestCase):
     @mock.patch.object(manager_utils, 'cleaning_error_handler', autospec=True)
     def test_apply_conf_post_reboot_cleaning_failed(
             self, mock_cleaning_error_handler):
-        data = [{'name': 'ProcTurboMode', 'value': 'Enabled'}]
+        data = [{'name': 'ProcTurboMode', 'value': 'Enabled',
+                 'registry': {'description': 'Turbo mode'}}]
         self.node.clean_step = {'priority': 100, 'interface': 'bios',
                                 'step': 'apply_configuration',
                                 'argsinfo': {'settings': data}}
@@ -345,7 +338,7 @@ class RedfishBiosTestCase(db_base.DbTestCase):
         attributes_after_reboot = {'ProcTurboMode': 'Disabled'}
         node = self.node
         driver_internal_info = node.driver_internal_info
-        driver_internal_info['post_config_reboot_requested'] = True
+        driver_internal_info['post_bios_reboot_requested'] = True
         driver_internal_info['requested_bios_attrs'] = requested_attrs
         self.node.driver_internal_info = driver_internal_info
         self.node.provision_state = states.CLEANING
@@ -355,8 +348,10 @@ class RedfishBiosTestCase(db_base.DbTestCase):
         mock_cleaning_error_handler.assert_called_once()
 
     def test_apply_conf_post_reboot_deploying(self):
-        data = [{'name': 'ProcTurboMode', 'value': 'Disabled'},
-                {'name': 'NicBoot1', 'value': 'NetworkBoot'}]
+        data = [{'name': 'ProcTurboMode', 'value': 'Enabled',
+                 'registry': {'description': 'Turbo mode'}},
+                {'name': 'NicBoot1', 'value': 'NetworkBoot',
+                 'registry': {'description': 'Boot off network'}}]
         self.node.deploy_step = {'priority': 100, 'interface': 'bios',
                                  'step': 'apply_configuration',
                                  'argsinfo': {'settings': data}}
@@ -364,7 +359,7 @@ class RedfishBiosTestCase(db_base.DbTestCase):
                            'NicBoot1': 'NetworkBoot'}
         node = self.node
         driver_internal_info = node.driver_internal_info
-        driver_internal_info['post_config_reboot_requested'] = True
+        driver_internal_info['post_bios_reboot_requested'] = True
         driver_internal_info['requested_bios_attrs'] = requested_attrs
         self.node.driver_internal_info = driver_internal_info
         self.node.save()
@@ -373,7 +368,8 @@ class RedfishBiosTestCase(db_base.DbTestCase):
     @mock.patch.object(manager_utils, 'deploying_error_handler', autospec=True)
     def test_apply_conf_post_reboot_deploying_failed(
             self, mock_deploying_error_handler):
-        data = [{'name': 'ProcTurboMode', 'value': 'Enabled'}]
+        data = [{'name': 'ProcTurboMode', 'value': 'Enabled',
+                 'registry': {'description': 'Turbo mode'}}]
         self.node.deploy_step = {'priority': 100, 'interface': 'bios',
                                  'step': 'apply_configuration',
                                  'argsinfo': {'settings': data}}
@@ -381,7 +377,7 @@ class RedfishBiosTestCase(db_base.DbTestCase):
         attributes_after_reboot = {'ProcTurboMode': 'Disabled'}
         node = self.node
         driver_internal_info = node.driver_internal_info
-        driver_internal_info['post_config_reboot_requested'] = True
+        driver_internal_info['post_bios_reboot_requested'] = True
         driver_internal_info['requested_bios_attrs'] = requested_attrs
         self.node.driver_internal_info = driver_internal_info
         self.node.provision_state = states.DEPLOYWAIT
@@ -432,7 +428,7 @@ class RedfishBiosTestCase(db_base.DbTestCase):
                                   shared=False) as task:
             attributes = mock_get_system(task.node).bios.attributes
             task.node.driver_internal_info[
-                'post_config_reboot_requested'] = True
+                'post_bios_reboot_requested'] = True
             task.node.driver_internal_info[
                 'requested_bios_attrs'] = requested_attrs
             task.driver.bios._check_bios_attrs = mock.MagicMock()
@@ -527,3 +523,208 @@ class RedfishBiosTestCase(db_base.DbTestCase):
             bios.set_attributes.assert_called_once_with(
                 {s['name']: s['value'] for s in settings},
                 apply_time=None)
+
+    @mock.patch.object(redfish_boot.RedfishVirtualMediaBoot, 'prepare_ramdisk',
+                       spec_set=True, autospec=True)
+    @mock.patch.object(deploy_utils, 'build_agent_options', autospec=True)
+    @mock.patch.object(redfish_utils, 'get_system', autospec=True)
+    @mock.patch.object(manager_utils, 'node_power_action', autospec=True)
+    def test_apply_configuration_no_apply_time_attr(
+            self, mock_power_action, mock_get_system, mock_build_agent_options,
+            mock_prepare):
+        settings = [{'name': 'ProcTurboMode', 'value': 'Disabled'},
+                    {'name': 'NicBoot1', 'value': 'NetworkBoot'}]
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=False) as task:
+            bios = mock_get_system(task.node).bios
+            del bios.supported_apply_times
+
+            task.driver.bios.apply_configuration(task, settings)
+
+            bios.set_attributes.assert_called_once_with(
+                {s['name']: s['value'] for s in settings},
+                apply_time=None)
+
+
+@mock.patch('oslo_utils.eventletutils.EventletEvent.wait',
+            lambda *args, **kwargs: None)
+class RedfishBiosRegistryTestCase(db_base.DbTestCase):
+
+    def setUp(self):
+        super(RedfishBiosRegistryTestCase, self).setUp()
+        self.config(enabled_bios_interfaces=['redfish'],
+                    enabled_hardware_types=['redfish'],
+                    enabled_power_interfaces=['redfish'],
+                    enabled_boot_interfaces=['redfish-virtual-media'],
+                    enabled_management_interfaces=['redfish'])
+        self.node = obj_utils.create_test_node(
+            self.context, driver='redfish', driver_info=INFO_DICT)
+
+        self.settings = {'SystemModelName': 'UltraSumma',
+                         'DcuStreamPrefetcher': 'Enabled',
+                         'BootDelay': 10}
+
+        class AttributeField():
+
+            def __init__(self):
+                self.name = None
+                self.allowable_values = None
+                self.attribute_type = None
+                self.lower_bound = None
+                self.max_length = None
+                self.min_length = None
+                self.read_only = None
+                self.reset_required = None
+                self.type = None
+                self.unique = None
+                self.upper_bound = None
+
+        class AttributeRegistryEntryField():
+
+            def __init__(self, num_entries):
+                self.attributes = []
+                for _ in range(num_entries):
+                    self.attributes.append(AttributeField())
+
+        class AttributeRegistryTest():
+
+            def __init__(self, num_entries):
+                self.registry_entries = AttributeRegistryEntryField(
+                    num_entries)
+
+        self.registry = AttributeRegistryTest(4)
+        self.registry.registry_entries.attributes[0].name = "SystemModelName"
+        self.registry.registry_entries.attributes[0].attribute_type = "String"
+        self.registry.registry_entries.attributes[0].max_length = 32
+        self.registry.registry_entries.attributes[0].read_only = True
+        self.registry.registry_entries.attributes[0].unique = True
+        self.registry.registry_entries.attributes[1].name =\
+            "DcuStreamPrefetcher"
+        self.registry.registry_entries.attributes[1].attribute_type =\
+            "Enumeration"
+        self.registry.registry_entries.attributes[1].read_only = False
+        self.registry.registry_entries.attributes[1].allowable_values =\
+            [{'ValueName': 'Enabled', 'ValueDisplayName': 'Enabled'},
+             {'ValueDisplayName': 'Disabled'},
+             {'Invalid': 'banana'}]
+        self.registry.registry_entries.attributes[2].name = "BootDelay"
+        self.registry.registry_entries.attributes[2].attribute_type = "Integer"
+        self.registry.registry_entries.attributes[2].lower_bound = 5
+        self.registry.registry_entries.attributes[2].upper_bound = 30
+        self.registry.registry_entries.attributes[2].reset_required = True
+        self.registry.registry_entries.attributes[3].name = "SomeAttribute"
+        self.registry.registry_entries.attributes[3].attribute_type = "String"
+        self.registry.registry_entries.attributes[3].max_length = 32
+        self.registry.registry_entries.attributes[3].read_only = True
+
+        self.expected_no_registry = [
+            {'name': 'SystemModelName', 'value': 'UltraSumma'},
+            {'name': 'DcuStreamPrefetcher', 'value': 'Enabled'},
+            {'name': 'BootDelay', 'value': 10}]
+
+    @mock.patch.object(objects, 'BIOSSettingList', autospec=True)
+    @mock.patch.object(redfish_utils, 'get_system', autospec=True)
+    def test_cache_bios_registry_save(self, mock_get_system,
+                                      mock_setting_list):
+        create_list = []
+        update_list = []
+        delete_list = []
+        nochange_list = []
+        mock_setting_list.sync_node_setting.return_value = (
+            create_list, update_list, delete_list, nochange_list
+        )
+
+        expected = [{'name': 'SystemModelName', 'value': 'UltraSumma',
+                     'allowable_values': None, 'lower_bound': None,
+                     'max_length': 32, 'min_length': None, 'read_only': True,
+                     'reset_required': None, 'attribute_type': 'String',
+                     'unique': True, 'upper_bound': None},
+                    {'name': 'DcuStreamPrefetcher', 'value': 'Enabled',
+                     'allowable_values': ['Enabled', 'Disabled'],
+                     'lower_bound': None, 'max_length': None,
+                     'min_length': None, 'read_only': False,
+                     'reset_required': None, 'attribute_type':
+                     'Enumeration', 'unique': None, 'upper_bound': None},
+                    {'name': 'BootDelay', 'value': 10,
+                     'allowable_values': None, 'lower_bound': 5,
+                     'max_length': None, 'min_length': None, 'read_only': None,
+                     'reset_required': True, 'attribute_type': 'Integer',
+                     'unique': None, 'upper_bound': 30}]
+
+        mock_get_system.return_value.bios.attributes = self.settings
+        mock_get_system.return_value.bios.get_attribute_registry.\
+            return_value = self.registry
+
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=True) as task:
+            task.driver.bios.cache_bios_settings(task)
+            mock_get_system.assert_called_once_with(task.node)
+            mock_setting_list.sync_node_setting.assert_called_once_with(
+                task.context, task.node.id, expected)
+
+    @mock.patch.object(objects, 'BIOSSettingList', autospec=True)
+    @mock.patch.object(redfish_utils, 'get_system', autospec=True)
+    def test_cache_empty_bios_registry(self, mock_get_system,
+                                       mock_setting_list):
+        create_list = []
+        update_list = []
+        delete_list = []
+        nochange_list = []
+        mock_setting_list.sync_node_setting.return_value = (
+            create_list, update_list, delete_list, nochange_list
+        )
+
+        mock_get_system.return_value.bios.attributes = self.settings
+        mock_get_system.return_value.bios.get_attribute_registry.\
+            return_value = {}
+
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=True) as task:
+            task.driver.bios.cache_bios_settings(task)
+            mock_setting_list.sync_node_setting.assert_called_once_with(
+                task.context, task.node.id, self.expected_no_registry)
+
+    @mock.patch.object(objects, 'BIOSSettingList', autospec=True)
+    @mock.patch.object(redfish_utils, 'get_system', autospec=True)
+    def test_cache_no_bios_registry(self, mock_get_system,
+                                    mock_setting_list):
+        create_list = []
+        update_list = []
+        delete_list = []
+        nochange_list = []
+        mock_setting_list.sync_node_setting.return_value = (
+            create_list, update_list, delete_list, nochange_list
+        )
+
+        mock_get_system.return_value.bios.attributes = self.settings
+        mock_get_system.return_value.bios.get_attribute_registry.\
+            return_value = None
+
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=True) as task:
+            task.driver.bios.cache_bios_settings(task)
+            mock_setting_list.sync_node_setting.assert_called_once_with(
+                task.context, task.node.id, self.expected_no_registry)
+
+    @mock.patch.object(objects, 'BIOSSettingList', autospec=True)
+    @mock.patch.object(redfish_utils, 'get_system', autospec=True)
+    def test_cache_exception_bios_registry(self, mock_get_system,
+                                           mock_setting_list):
+        create_list = []
+        update_list = []
+        delete_list = []
+        nochange_list = []
+        mock_setting_list.sync_node_setting.return_value = (
+            create_list, update_list, delete_list, nochange_list
+        )
+
+        mock_get_system.return_value.bios.attributes = self.settings
+
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=True) as task:
+            bios = mock_get_system(task.node).bios
+            bios.reset_bios.side_effect = sushy.exceptions.SushyError
+
+            task.driver.bios.cache_bios_settings(task)
+            mock_setting_list.sync_node_setting.assert_called_once_with(
+                task.context, task.node.id, self.expected_no_registry)
