@@ -14,8 +14,8 @@
 
 from unittest import mock
 
-from oslo_utils import importutils
 from oslo_utils import units
+import sushy
 
 from ironic.common import exception
 from ironic.common import states
@@ -29,8 +29,6 @@ from ironic.tests.unit.db import base as db_base
 from ironic.tests.unit.db import utils as db_utils
 from ironic.tests.unit.objects import utils as obj_utils
 
-sushy = importutils.try_import('sushy')
-
 INFO_DICT = db_utils.get_test_redfish_info()
 
 
@@ -55,7 +53,10 @@ def _mock_volume(identity, raid_type=None,
         raid_type=raid_type,
         capacity_bytes=capacity_bytes
     )
-    volume.name = 'Volume ' + identity
+    if volume_name:
+        volume.name = volume_name
+    else:
+        volume.name = 'Volume ' + identity
     # Mocking Immediate that does not return anything
     volume.delete.return_value = None
     return volume
@@ -108,13 +109,6 @@ class RedfishRAIDTestCase(db_base.DbTestCase):
                                  mock_drives}
         self.physical_disks = mock_drives
 
-    @mock.patch.object(redfish_raid, 'sushy', None)
-    def test_loading_error(self, mock_get_system):
-        self.assertRaisesRegex(
-            exception.DriverLoadError,
-            'Unable to import the sushy library',
-            redfish_raid.RedfishRAID)
-
     def test__max_volume_size_bytes_raid0(self, mock_get_system):
         spans = redfish_raid._calculate_spans('0', 3)
         max_size = redfish_raid._max_volume_size_bytes(
@@ -277,13 +271,14 @@ class RedfishRAIDTestCase(db_base.DbTestCase):
                 {
                     'size_gb': 100,
                     'raid_level': '5',
-                    'is_root_volume': True
+                    'is_root_volume': True,
+                    'volume_name': 'test-volume'
                 }
             ]
         }
-        created_volumes = [_mock_volume(
-            '1', raid_type=sushy.RAIDType.RAID5,
-            capacity_bytes=100 * units.Gi)]
+        created_volumes = [_mock_volume('1', raid_type=sushy.RAIDType.RAID5,
+                                        capacity_bytes=100 * units.Gi,
+                                        volume_name='test-volume')]
         volumes = mock.MagicMock()
         # Called after volumes created
         volumes.get_members.return_value = created_volumes
@@ -312,7 +307,8 @@ class RedfishRAIDTestCase(db_base.DbTestCase):
                         {'@odata.id': pre + self.drive_id2},
                         {'@odata.id': pre + self.drive_id3}
                     ]
-                }
+                },
+                'Name': 'test-volume'
             }
             self.mock_storage.volumes.create.assert_called_once_with(
                 expected_payload, apply_time=sushy.APPLY_TIME_IMMEDIATE)
@@ -326,7 +322,7 @@ class RedfishRAIDTestCase(db_base.DbTestCase):
             self.assertEqual(
                 [{'controller': 'RAID controller 1',
                   'id': '1',
-                  'name': 'Volume 1',
+                  'name': 'test-volume',
                   'raid_level': '5',
                   'size_gb': 100}],
                 task.node.raid_config['logical_disks'])
@@ -693,7 +689,8 @@ class RedfishRAIDTestCase(db_base.DbTestCase):
                         {'@odata.id': pre + self.drive_id3},
                         {'@odata.id': pre + self.drive_id4}
                     ]
-                }
+                },
+                'Name': 'root_volume'
             }
             expected_payload2 = {
                 'RAIDType': 'RAID5',
@@ -704,7 +701,8 @@ class RedfishRAIDTestCase(db_base.DbTestCase):
                         {'@odata.id': pre + self.drive_id3},
                         {'@odata.id': pre + self.drive_id4}
                     ]
-                }
+                },
+                'Name': 'data_volume'
             }
             self.assertEqual(
                 self.mock_storage.volumes.create.call_count, 2)
@@ -1078,11 +1076,7 @@ class RedfishRAIDTestCase(db_base.DbTestCase):
             'RAIDType': 'RAID1',
             'CapacityBytes': capacity_bytes,
             'Links': {
-                'Drives': [
-                    {
-                        '@odata.id': pre + drive_id
-                    }
-                ]
+                'Drives': [{'@odata.id': pre + drive_id}]
             }
         }
         with task_manager.acquire(self.context, self.node.uuid,
@@ -1143,7 +1137,9 @@ class RedfishRAIDTestCase(db_base.DbTestCase):
         nonraid_controller = mock.Mock()
         nonraid_controller.raid_types = []
         nonraid_storage = mock.MagicMock()
-        nonraid_storage.storage_controllers = [nonraid_controller]
+        nonraid_storage.controllers = mock.MagicMock()
+        nonraid_storage.controllers.get_members.return_value = [
+            nonraid_controller]
         nonraid_storage.drives = [_mock_drive(
             identity='Drive1', block_size_bytes=512,
             capacity_bytes=899527000000,
@@ -1168,6 +1164,7 @@ class RedfishRAIDTestCase(db_base.DbTestCase):
         nonraid_controller = mock.Mock()
         nonraid_controller.raid_types = []
         nonraid_storage = mock.MagicMock()
+        nonraid_storage.controllers = None
         nonraid_storage.storage_controllers = [nonraid_controller]
         nonraid_storage.drives = mock.Mock()
 
@@ -1182,6 +1179,73 @@ class RedfishRAIDTestCase(db_base.DbTestCase):
             self.assertEqual(storage, self.mock_storage)
             nonraid_storage.drives.assert_not_called()
 
+    def test__get_controller(self, mock_get_system):
+        nonraid_controller = mock.Mock()
+        nonraid_controller.raid_types = []
+        nonraid_storage = mock.MagicMock()
+        nonraid_storage.get_members.return_value = [nonraid_controller]
+        nonraid_storage.controllers = mock.MagicMock()
+        nonraid_storage.controllers.get_members.return_value = [
+            nonraid_controller]
+        nonraid_storage.drives = [_mock_drive(
+            identity='Drive1', block_size_bytes=512,
+            capacity_bytes=899527000000,
+            media_type='HDD', name='Drive', protocol='SAS')]
+
+        mock_get_system.return_value.storage.get_members.return_value = [
+            nonraid_storage, self.mock_storage]
+
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=True) as task:
+            storage = redfish_raid._get_storage_controller(
+                task.node, mock_get_system.return_value, ['32ADF365C6C1B7BD'])
+
+            self.assertEqual(storage, self.mock_storage)
+
+    def test__get_first_controller_controllers(self, mock_get_system):
+        controller1 = mock.Mock(name="controller")
+        controller2 = mock.Mock(name="storage_controller")
+
+        storage = mock.MagicMock()
+        storage.get_members.return_value = [controller1]
+
+        storage.storage_controllers = mock.MagicMock()
+        storage.controllers = mock.MagicMock()
+
+        storage.controllers.get_members.return_value = [controller1]
+        storage.storage_controllers.return_value = [controller2]
+
+        result = redfish_utils.get_first_controller(storage)
+        self.assertEqual(result, controller1)
+
+    def test__get_first_controller_storage_controllers(self, mock_get_system):
+        storage = mock.Mock()
+        controller_mock = mock.Mock()
+        storage.storage_controllers = [controller_mock]
+        type(storage).controllers = mock.PropertyMock(
+            side_effect=sushy.exceptions.MissingAttributeError)
+
+        result = redfish_utils.get_first_controller(storage)
+        self.assertEqual(result, controller_mock)
+
+    def test__get_controller_no_controllers(self, mock_get_system):
+        storage = mock.Mock()
+        storage.get_members.return_value = None
+        storage.controllers.return_value = None
+        storage.controllers.get_members.return_value = None
+        storage.storage_controllers.return_value = None
+
+        result = redfish_utils.get_first_controller(storage)
+        self.assertIsNone(result)
+
+    def test__get_first_controller_empty(self, mock_get_system):
+        storage = mock.Mock()
+        storage.controllers = mock.Mock()
+        storage.controllers.get_members.return_value = []
+
+        result = redfish_utils.get_first_controller(storage)
+        self.assertIsNone(result)
+
     @mock.patch.object(redfish_utils, 'get_task_monitor', autospec=True)
     @mock.patch.object(redfish_raid.LOG, 'info', autospec=True)
     def test__raid_config_in_progress_success(
