@@ -17,36 +17,6 @@
 """This module provides mock 'specs' for third party modules that can be used
 when needing to mock those third party modules"""
 
-# python-dracclient
-DRACCLIENT_SPEC = (
-    'client',
-    'constants',
-    'exceptions',
-)
-
-DRACCLIENT_CLIENT_MOD_SPEC = (
-    'DRACClient',
-)
-
-DRACCLIENT_CONSTANTS_MOD_SPEC = (
-    'POWER_OFF',
-    'POWER_ON',
-    'REBOOT',
-    'RebootRequired',
-    'RaidStatus'
-)
-
-DRACCLIENT_CONSTANTS_REBOOT_REQUIRED_MOD_SPEC = (
-    'true',
-    'optional',
-    'false'
-)
-
-DRACCLIENT_CONSTANTS_RAID_STATUS_MOD_SPEC = (
-    'jbod',
-    'raid'
-)
-
 # sushy_oem_idrac
 SUSHY_OEM_IDRAC_MOD_SPEC = (
     'PHYSICAL_DISK_STATE_MODE_RAID',
@@ -95,9 +65,11 @@ SCCICLIENT_IRMC_SCCI_SPEC = (
     'get_virtual_fd_set_params_cmd',
     'get_essential_properties',
     'get_capabilities_properties',
+    'get_irmc_version_str',
 )
 SCCICLIENT_IRMC_ELCM_SPEC = (
     'backup_bios_config',
+    'elcm_request',
     'restore_bios_config',
     'set_secure_boot_mode',
 )
@@ -119,28 +91,3 @@ SCCICLIENT_VIOM_CONF_SPEC = (
 REDFISH_SPEC = (
     'redfish',
 )
-
-XCLARITY_SPEC = (
-    'client',
-    'states',
-    'exceptions',
-    'models',
-    'utils',
-)
-
-XCLARITY_CLIENT_CLS_SPEC = (
-)
-
-XCLARITY_STATES_SPEC = (
-    'STATE_POWERING_OFF',
-    'STATE_POWERING_ON',
-    'STATE_POWER_OFF',
-    'STATE_POWER_ON',
-)
-
-# python-ibmcclient
-IBMCCLIENT_SPEC = (
-    'connect',
-    'exceptions',
-    'constants',
-)
