@@ -17,7 +17,6 @@ import datetime
 import types
 from unittest import mock
 
-import iso8601
 from oslo_utils import timeutils
 from oslo_versionedobjects import base as object_base
 from oslo_versionedobjects import exception as object_exception
@@ -291,7 +290,7 @@ class _TestObject(object):
         self.assertEqual('updated', obj.bar)
 
     def test_base_attributes(self):
-        dt = datetime.datetime(1955, 11, 5, 0, 0, tzinfo=iso8601.UTC)
+        dt = datetime.datetime(1955, 11, 5, 0, 0, tzinfo=datetime.timezone.utc)
         datatime = fields.DateTimeField()
         obj = MyObj(self.context)
         obj.created_at = dt
@@ -676,29 +675,29 @@ class TestObject(_LocalTest, _TestObject):
 # version bump. It is an MD5 hash of the object fields and remotable methods.
 # The fingerprint values should only be changed if there is a version bump.
 expected_object_fingerprints = {
-    'Node': '1.35-aee8ecf5c4d0ed590eb484762aee7fca',
+    'Node': '1.41-baff7b2b06243d97448b720030b2e612',
     'MyObj': '1.5-9459d30d6954bffc7a9afd347a807ca6',
     'Chassis': '1.3-d656e039fd8ae9f34efc232ab3980905',
-    'Port': '1.10-67381b065c597c8d3a13c5dbc6243c33',
-    'Portgroup': '1.4-71923a81a86743b313b190f5c675e258',
-    'Conductor': '1.3-d3f53e853b4d58cae5bfbd9a8341af4a',
+    'Port': '1.11-97bf15b61224f26c65e90f007d78bfd2',
+    'Portgroup': '1.5-df4dc15967f67114d51176a98a901a83',
+    'Conductor': '1.4-a9703208fdab5fab8f1cec420be1b4a7',
     'EventType': '1.1-aa2ba1afd38553e3880c267404e8d370',
     'NotificationPublisher': '1.0-51a09397d6c0687771fb5be9a999605d',
-    'NodePayload': '1.15-86ee30dbf374be4cf17c5b501d9e2e7b',
+    'NodePayload': '1.17-4022bb737b058d426a7ff878b1875e5c',
     'NodeSetPowerStateNotification': '1.0-59acc533c11d306f149846f922739c15',
-    'NodeSetPowerStatePayload': '1.15-3c64b07a2b96c2661e7743b47ed43705',
+    'NodeSetPowerStatePayload': '1.17-bde6f731995024e718f42021e50ba7b4',
     'NodeCorrectedPowerStateNotification':
         '1.0-59acc533c11d306f149846f922739c15',
-    'NodeCorrectedPowerStatePayload': '1.15-59a224a9191cdc9f1acc2e0dcd2d3adb',
+    'NodeCorrectedPowerStatePayload': '1.17-6e1766fc690e0403bc0a323c8f6f12b7',
     'NodeSetProvisionStateNotification':
         '1.0-59acc533c11d306f149846f922739c15',
-    'NodeSetProvisionStatePayload': '1.16-c5a8eea43c514baf721fc61ce5d9d5a4',
+    'NodeSetProvisionStatePayload': '1.18-ac98c88d6dc8c6c924a415868f0a36e7',
     'VolumeConnector': '1.0-3e0252c0ab6e6b9d158d09238a577d97',
     'VolumeTarget': '1.0-0b10d663d8dae675900b2c7548f76f5e',
     'ChassisCRUDNotification': '1.0-59acc533c11d306f149846f922739c15',
     'ChassisCRUDPayload': '1.0-dce63895d8186279a7dd577cffccb202',
     'NodeCRUDNotification': '1.0-59acc533c11d306f149846f922739c15',
-    'NodeCRUDPayload': '1.13-8f673253ff8d7389897a6a80d224ac33',
+    'NodeCRUDPayload': '1.15-9168946f843edd5859464aaa40ad70e0',
     'PortCRUDNotification': '1.0-59acc533c11d306f149846f922739c15',
     'PortCRUDPayload': '1.4-9411a1701077ae9dc0aea27d6bf586fc',
     'NodeMaintenanceNotification': '1.0-59acc533c11d306f149846f922739c15',
@@ -711,7 +710,7 @@ expected_object_fingerprints = {
     'VolumeTargetCRUDPayload': '1.0-30dcc4735512c104a3a36a2ae1e2aeb2',
     'Trait': '1.0-3f26cb70c8a10a3807d64c219453e347',
     'TraitList': '1.0-33a2e1bb91ad4082f9f63429b77c1244',
-    'BIOSSetting': '1.0-fd4a791dc2139a7cc21cefbbaedfd9e7',
+    'BIOSSetting': '1.1-1137db88675a4e2d7f7bcc3a0d52345a',
     'BIOSSettingList': '1.0-33a2e1bb91ad4082f9f63429b77c1244',
     'Allocation': '1.1-38937f2854722f1057ec667b12878708',
     'AllocationCRUDNotification': '1.0-59acc533c11d306f149846f922739c15',
@@ -720,6 +719,16 @@ expected_object_fingerprints = {
     'DeployTemplateCRUDNotification': '1.0-59acc533c11d306f149846f922739c15',
     'DeployTemplateCRUDPayload': '1.0-200857e7e715f58a5b6d6b700ab73a3b',
     'Deployment': '1.0-ff10ae028c5968f1596131d85d7f5f9d',
+    'NodeHistory': '1.0-9b576c6481071e7f7eac97317fa29418',
+    'NodeInventory': '1.0-97692fec24e20ab02022b9db54e8f539',
+    'FirmwareComponent': '1.0-0e0720dab959e20247bbcfd5f28958c5',
+    'FirmwareComponentList': '1.0-33a2e1bb91ad4082f9f63429b77c1244',
+    'Runbook': '1.0-7a9c65b49b5f7b45686b6a674e703629',
+    'RunbookCRUDNotification': '1.0-59acc533c11d306f149846f922739c15',
+    'RunbookCRUDPayload': '1.0-f0c97f4ff29eb3401e53b34550a95e30',
+    'InspectionRule': '1.0-517185d327442696e408781343c2b83f',
+    'InspectionRuleCRUDNotification': '1.0-59acc533c11d306f149846f922739c15',
+    'InspectionRuleCRUDPayload': '1.0-85d1cf2105308534a630299a897bf562',
 }
 
 
@@ -976,6 +985,7 @@ class TestObjectSerializer(test_base.TestCase):
 class TestRegistry(test_base.TestCase):
     @mock.patch('ironic.objects.base.objects', autospec=True)
     def test_hook_chooses_newer_properly(self, mock_objects):
+        del mock_objects.MyObj
         reg = base.IronicObjectRegistry()
         reg.registration_hook(MyObj, 0)
 
@@ -992,6 +1002,7 @@ class TestRegistry(test_base.TestCase):
 
     @mock.patch('ironic.objects.base.objects', autospec=True)
     def test_hook_keeps_newer_properly(self, mock_objects):
+        del mock_objects.MyObj
         reg = base.IronicObjectRegistry()
         reg.registration_hook(MyObj, 0)
 
